/*
 * @(#)file      XMLMibValidationDOMParser.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.7
 * @(#)lastedit  04/05/10
 * 
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package validator;

// JAXP packages
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.w3c.dom.*;

import java.io.File;
import java.net.URL;
import java.net.URI;
public class XMLMibValidationDOMParser extends XMLSchemaDOMParser{
    
    static final ValidatorLogger logger = 
	new ValidatorLogger(XMLMibValidationDOMParser.class);
    static final String SCHEMA_RESOURCE_NAME = "validator/mibvalidation.xsd";
    public XMLMibValidationDOMParser(File schema) {
	super(schema, new MibErrorHandler(logger));
    }

    public SnmpMibValidator parseMibValidation(File validation) 
	throws Exception {
	
	Document doc = parse(validation);
	
        // Handle the DOM tree
	XMLMibValidationHandler handler = new XMLMibValidationHandler(logger);
	SnmpMibValidator validator = new SnmpMibValidator();
	handler.handle(doc, validator);
	return validator;
    }

    private static File getRessourceFile() {	
	URL url = null;
	try {
	    url = 
		XMLMibValidationDOMParser.class.getClassLoader().getResource(SCHEMA_RESOURCE_NAME);
	    if(url == null) throw new Exception("URL is null");
	    System.out.println("URL 1: " + url);
	    System.out.println("URL 2: " + url.toExternalForm());
	    URI uri = new URI(url.toExternalForm());
	    System.out.println("URI 1: " + uri);
	    if(uri == null) throw new Exception("URI is null");
	    
	    //return new File(uri);
	    return new File("./src/validator/mibvalidation.xsd");
	    
	}catch(Exception e) {
	    e.printStackTrace();
	    logger.error("","Error accessing Schema File resource [" +
			 SCHEMA_RESOURCE_NAME +"]");  
	}
	return null;
    }
    
    // Error handler to report errors and warnings
    private static class MibErrorHandler implements ErrorHandler {
        /** Error handler output goes here */
        private ValidatorLogger logger;
	
        MibErrorHandler(ValidatorLogger logger) {
            this.logger = logger;
        }

        /**
         * Returns a string describing parse exception details
         */
        private String getParseExceptionInfo(SAXParseException spe) {
            String systemId = spe.getSystemId();
            if (systemId == null) {
                systemId = "null";
            }
            String info = "URI=" + systemId +
                " Line=" + spe.getLineNumber() +
                ": " + spe.getMessage();
            return info;
        }

        // The following methods are standard SAX ErrorHandler methods.
        // See SAX documentation for more info.

        public void warning(SAXParseException spe) throws SAXException {
            logger.info("", "Warning: " + getParseExceptionInfo(spe));
        }
        
        public void error(SAXParseException spe) throws SAXException {
            String message = "Error: " + getParseExceptionInfo(spe);
            throw new SAXException(message);
        }

        public void fatalError(SAXParseException spe) throws SAXException {
            String message = "Fatal Error: " + getParseExceptionInfo(spe);
            throw new SAXException(message);
        }
    }
}
