/*
 * @(#)file      SnmpProxiesReader.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.12
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.internal.v3proxy;

import java.util.Enumeration;

import com.sun.jdmk.trace.Trace;
import com.sun.management.snmp.agent.SnmpProxy;
import com.sun.management.internal.snmp.SnmpPersistRowFileConsumer;
import com.sun.management.snmp.SnmpEngineId;
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpEngine;
import com.sun.management.snmp.rfc2573.target.SnmpParamsData;
import com.sun.management.snmp.rfc2573.v3proxy.SnmpProxyData;
import com.sun.management.snmp.rfc2573.target.SnmpTargetLcd;
import com.sun.management.snmp.rfc2573.internal.SnmpPersistRowFileReader;

class ProxiesConsumer implements SnmpPersistRowFileConsumer {
    //Define contexts
    static final int PROXY_NAME = 0;
    static final int TYPE = 1;
    static final int CTX_ENGINEID = 2;
    static final int CTX_NAME = 3;
    static final int PARAMS_IN = 4;
    static final int SINGLE_OUT = 5;
    static final int MULTIPLE_OUT = 6;
    static final int STORAGE_TYPE = 7;

    SnmpProxyData proxy = new SnmpProxyData();
    SnmpProxiesReader reader = null;
    ProxiesConsumer(SnmpProxiesReader reader) {
	this.reader = reader;
    }
    void reset() {
	proxy.proxyName = null;
	proxy.type = -1;
	proxy.contextEngineId = null;
	proxy.contextName = null;
	proxy.paramsIn = null;
	proxy.singleOut = null;
	proxy.multipleOut = null;
	proxy.storageType = -1;
	proxy.proxy = null;
    }
    public Object rowBegin(String row, int line) {
	reset(); 
	return proxy;
    }
    
    public void rowEnd(Object rowContext, String row, int line) 
	throws Exception  {
	SnmpProxyData data = (SnmpProxyData) rowContext;
	reader.rowRead(data, row, line);
    }
    
    public void treatToken(String row,
			   int lineNumber,
			   String token,
			   int context,
			   Object rowContext,
			   boolean err) throws Exception {
	SnmpProxyData proxy = (SnmpProxyData) rowContext;
	if( err || (proxy == null) ) 
	    throw new Exception("Badly formed param entry. Line ["+
				lineNumber+"], data [" + row+"]");
	
	switch(context) {
	case PROXY_NAME:
	    proxy.proxyName = token;
	    break;
	case TYPE:
	    proxy.type = Integer.parseInt(token);
	    break;
	case CTX_ENGINEID:
	    proxy.contextEngineId = SnmpEngineId.createEngineId(token);
	    break;
	case CTX_NAME:
	    proxy.contextName = token;
	    break;
	case PARAMS_IN:
	    proxy.paramsIn = reader.targetLcd.getParams(token);
	    if(proxy.paramsIn == null)
		throw new IllegalArgumentException("WARNING. PROXY NOT CREATED. Referenced IN parameter ["+ token +"] is not found."+ " Line :" + 
						   lineNumber + 
						   ".\n" + 
						   " Row : [" +
						   row +"]");
	    break;
	case SINGLE_OUT:
	    proxy.singleOut = reader.targetLcd.getTarget(token);
	    if(proxy.singleOut == null)
		throw new IllegalArgumentException("WARNING. PROXY NOT CREATED. Referenced OUT target ["+ token +"] is not found." + " Line :" + 
						   lineNumber + 
						   ".\n" + 
						   " Row : [" +
						   row +"]");
	    break;
	case MULTIPLE_OUT:
	    proxy.multipleOut = token;
	    break;
	case STORAGE_TYPE:
	    proxy.storageType = Integer.parseInt(token);
	    break;
	default:
	}
    }
}

class SnmpProxiesReader extends SnmpPersistRowFileReader {
    SnmpProxyFactory factory = null;
    SnmpEngine engine = null;
    SnmpTargetLcd targetLcd = null;
    SnmpJdmkProxyLcd proxyLcd = null;
    public SnmpProxiesReader(SnmpEngine engine,
			     SnmpJdmkProxyLcd proxyLcd,
			     SnmpProxyFactory factory,
			     String file, 
			     SnmpTargetLcd targetLcd) {
	super(file, "proxiesEntry");
	this.proxyLcd = proxyLcd;
	this.engine = engine;
	this.targetLcd = targetLcd;
	this.factory = factory;
	init(new ProxiesConsumer(this));
    }
    
    public SnmpProxyData getProxy(String name) {
	return (SnmpProxyData) getObject(name);
    }

    public Enumeration getProxies() {
	return getObjects();
    }

    public SnmpProxyData findProxy(SnmpEngineId ctxEngineId,
				   String ctxName,
				   int type,
				   int pduType,
				   int mpm, 
				   int securityModel,
				   String securityName,
				   int securityLevel) {
	SnmpProxyData ret = null;
	if(isDebugOn()) {
	    debug("findProxy", " ctxEngineId :" + 
		  ctxEngineId + " ctxName :" + 
		  ctxName + " type :" + type + " pduType :" + 
		  pduType + " mpm :" + mpm + " securityModel:" + 
		  securityModel + " securityName :" + 
		  securityName + " securityLevel: " + securityLevel);
	}

	for(Enumeration e = getObjects(); e.hasMoreElements();) {
	    SnmpProxyData data = (SnmpProxyData) e.nextElement();
	    if(isDebugOn()) {
		debug("findProxy", " data : " + data);
	    }
	    if(data.contextEngineId.equals(ctxEngineId)) {
		if(data.contextName.equals(ctxName)) {
		    if(data.type == type) {
			if(data.paramsIn.mpm == mpm) {
			    if(data.paramsIn.securityName.equals(securityName)) {
				if(data.paramsIn.securityModel == securityModel) {
				    if(SnmpParamsData.translateMIBSecurityLevel(data.paramsIn.securityLevel) == securityLevel) {
					if((pduType == SnmpDefinitions.pduGetRequestPdu ) ||
					   (pduType == SnmpDefinitions.pduSetRequestPdu) ||
					   (pduType == SnmpDefinitions.pduGetNextRequestPdu) ||
					   (pduType == SnmpDefinitions.pduGetBulkRequestPdu) ) {
					    if(ret == null) {
						if(isDebugOn())
						    debug("findProxy", "FOUND first proxy for get/set/...:"  + data);
						ret = data;
					    }
					    else {
						if(String.CASE_INSENSITIVE_ORDER.compare(data.proxyName, ret.proxyName) < 0) {
						    if(isDebugOn())
							debug("findProxy", "FOUND proxy for get/set/...:"  + data);
						    ret = data;
						}
						else {
						    if(isDebugOn())
							debug("findProxy", "FOUND proxybut bigger name :"  + data);
						}
					    }
					}
					else {
					    if(isDebugOn())
						debug("findProxy", "FOUND proxy for imform trap:"  + data);
					    ret = data;
					    break;
					}
				    }
				    else {
					if(isDebugOn())
					    debug("findProxy"," mistmatch securityLevel. Received :" + securityLevel + " Present : " + data.paramsIn.securityLevel);
					continue;
				    }
				}
				else {
				    if(isDebugOn())
					debug("findProxy"," mistmatch securityModel. Received :" + securityModel + " Present : " + data.paramsIn.securityModel);
				    continue;
				}
			    }
			    else {
				if(isDebugOn())
				    debug("findProxy"," mistmatch securityName. Received :" + securityName + " Present : " + data.paramsIn.securityName);
				continue;
			    }
			}
			else {
			    if(isDebugOn())
				debug("findProxy"," mistmatch mpm. Received :" + mpm + " Present : " + data.paramsIn.mpm);
			    continue;
			}
		    }
		    else {
			if(isDebugOn())
			    debug("findProxy"," mistmatch type. Received :" + type + " Present : " + data.type);
			continue;
		    }
		}
		else {
		    if(isDebugOn())
			debug("findProxy"," mistmatch contextName. Received :" + ctxName + " Present : " + data.contextName);
		    continue;
		}
	    }
	    else {
		if(isDebugOn())
		    debug("findProxy"," mistmatch ContextEngineId. Received :" + ctxEngineId + " Present : " + data.contextEngineId);
		continue;
	    } 
	}
	
	return ret;
    }
    
    public void removeProxy(String name) throws IllegalArgumentException {
	 removeObject(name);
	 flushInFile();
    }

    public void addProxy(String proxyName,
			 int type,
			 SnmpEngineId contextEngineId,
			 String contextName,
			 String paramsIn,
			 String singleOut,
			 String multipleOut,
			 int storageType) throws IllegalArgumentException {
	SnmpProxyData data = new SnmpProxyData();
	data.proxyName = proxyName;
	data.type = type;
	data.contextEngineId = contextEngineId;
	data.contextName = contextName;
	data.multipleOut = multipleOut;
	data.storageType = storageType;
	data.paramsIn = targetLcd.getParams(paramsIn);
	if(data.paramsIn == null) {
	    if(isDebugOn()) {
		debug("addProxy", 
		      "WARNING. PROXY NOT CREATED. Referenced IN parameter ["+  paramsIn +"] is not found.");
	    }
	    throw new IllegalArgumentException("Referenced IN parameter ["+  paramsIn +"] is not found.");
	}
	data.singleOut = targetLcd.getTarget(singleOut);
	if(data.singleOut == null) {
	    if(isDebugOn()) {
		debug("addProxy", 
		      "WARNING. PROXY NOT CREATED. Referenced OUT target ["+ singleOut +"] is not found." );
	    }
	    throw new IllegalArgumentException("Referenced OUT target ["+ singleOut +"] is not found." );
	}
	data.storageType = storageType;
	try {
	    addProxy(data);
	}catch(Exception e) {
	    throw new IllegalArgumentException(e.getMessage());
	}
    }
    
    void addProxy(SnmpProxyData data) throws Exception {
	SnmpAgentProxyImpl agent = new SnmpAgentProxyImpl(data,
							  factory,
							  engine);
	data.proxy = agent;
	proxyLcd.addContextEngineIdProxy(data.contextEngineId, data.type);
	if(isDebugOn())
	    debug("addProxy", "New proxy added for : " + 
		  data.contextEngineId);
	
	putObject(data.proxyName, data);
    }
    
    void rowRead(SnmpProxyData d, String row, int line) throws IllegalArgumentException {
	try {
	    addProxy((SnmpProxyData) d.clone());
	}catch(Exception e) {
	    if(isDebugOn())
		debug("rowRead", e.toString() + ". Line : "+ line + " Row :" + row);
	    return;
	}
    }
    
    // TRACES & DEBUG
    //---------------
    
    boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
    }

    void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
    }

    void debug(String clz, String func, Throwable exception) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
    }

    void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
    
    void debug(String func, Throwable exception) {
        debug(dbgTag, func, exception);
    }
    
    String dbgTag = "SnmpProxiesReader"; 
}
