/*
 * @(#)file      SnmpV3AppliMibRegistration.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.11
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.manager;

// Jdmk import
import com.sun.jdmk.trace.Trace;
import com.sun.jdmk.TraceManager;

// JMX RI import
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.manager.SnmpSession;
import com.sun.management.snmp.SnmpEngineId;


// Contrib import
import com.sun.management.snmp.rfc2573.manager.notification.SnmpRfc2573NotificationManager;
import com.sun.management.snmp.rfc2573.manager.target.SnmpRfc2573TargetManager;
import com.sun.management.snmp.rfc2573.manager.v3proxy.SnmpRfc2573ProxyManager;

/**
 * This class allow to register/unregister targets
 * according to the RFC2573
 */
public class SnmpV3AppliMibRegistration {

    public static void registerParams(SnmpSession session,
			       String paramsIndex,
			       int    processingModel,
			       int    securityModel,
			       String securityName,
			       int    securityLevel) 
	throws SnmpStatusException {
	SnmpRfc2573TargetManager.registerParams(
						session,
						paramsIndex,
						processingModel,
						securityModel,
						securityName,
						securityLevel,
						3);
	if (isDebugOn())
	    debug("registerParams", " ........ done");
    }

	
    /** 
     * Unregister Params
     */
    public static void unregisterParams(SnmpSession session,
				 String paramsIndex) 
	throws SnmpStatusException {
	SnmpRfc2573TargetManager.unregisterParams(session,
						  paramsIndex);
	if (isDebugOn())
	    debug("unregisterParams", " ........ done");
    }

    /**
     * Allows to register a target to a proxy forwarder
     * using the SnmpSession according to the RFC2573
     */
    public static void registerTarget(SnmpSession session,
			       String targetIndex,
			       String targetDomain,
			       String targetAddress,
			       int    targetTimeout,
			       int    targetRetry,
			       String targetTagList,
			       String paramsIndex)

	 throws SnmpStatusException {
	
	SnmpRfc2573TargetManager.registerTarget(session,
						targetIndex,
						targetDomain,
						targetAddress,
						targetTimeout,
						targetRetry,
						targetTagList,
						paramsIndex,
						3);
	if (isDebugOn())
	    debug("registerTarget", " ........ done");
    }
    
    /**
     * Allows to unregister a target to a proxy forwarder
     * using the SnmpSession according to the RFC2573
     */
    public static void unregisterTarget(SnmpSession session,
				 String targetIndex)
        throws SnmpStatusException {
        SnmpRfc2573TargetManager.unregisterTarget(session,
						  targetIndex);
	if (isDebugOn())
	    debug("unregisterTarget", " ........ done");
    }
    
    /**
     * Allows to register a target to a notification originator
     * using the SnmpSession according to the RFC2573
     */
    public static void registerForTrap(SnmpSession session,
				String notifIndex,
				String tag)
        throws SnmpStatusException {
        SnmpRfc2573NotificationManager.registerNotif(session,
						     notifIndex, 
						     tag, 
						     1, 
						     3);
	if (isDebugOn())
	    debug("registerNotif", " ........ done");
    }
    
    /**
     * Allows to unregister a target from a notification originator
     * using the SnmpSession according to the RFC2573
     */
    public static void unregisterForTrap(SnmpSession session,
					 String notifIndex) 
        throws SnmpStatusException {
        SnmpRfc2573NotificationManager.unregisterNotif(session,
						       notifIndex);
	if (isDebugOn())
	    debug("unregisterForTrap", " ........ done");
    }
    
    /**
     * Allows to register a target to a notification originator
     * using the SnmpSession according to the RFC2573
     */
    public static void registerForInform(SnmpSession session,
				  String notifIndex,
				  String tag) 
	throws SnmpStatusException {
        SnmpRfc2573NotificationManager.registerNotif(session,
						     notifIndex, 
						     tag, 
						     2, 
						     3);
	if (isDebugOn())
	    debug("registerForInform", " ........ done");
    }

    /**
     * Allows to unregister a target from a notification originator
     * using the SnmpSession according to the RFC2573
     */
    public static void unregisterForInform(SnmpSession session,
				    String notifIndex) 
         throws SnmpStatusException {
        SnmpRfc2573NotificationManager.unregisterNotif(session, notifIndex);
	if (isDebugOn())
	    debug("unregisterForInform", " ........ done");
    }

    public static void registerProxy(SnmpSession session,
				     String proxyIndex, 
				     int requestType,
				     SnmpEngineId id,
				     String contextName,
				     String paramsIn,
				     String singleOut,
				     String multipleOut) 
	throws SnmpStatusException {

	SnmpRfc2573ProxyManager.registerProxy(
					      session,
					      proxyIndex, 
					      requestType,
					      id,
					      contextName,
					      paramsIn,
					      singleOut,
					      multipleOut,
					      3);
	if (isDebugOn())
	    debug("registerProxy", " ........ done");
	
    }

    public static void unregisterProxy(SnmpSession session,
				       String proxyIndex) 
	throws SnmpStatusException {
	SnmpRfc2573ProxyManager.unregisterProxy(session,
				     proxyIndex);
    }

    static boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    static void trace(String clz, String func, String info) {
        if (isTraceOn()) {
            Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
        }
    }

    static void trace(String func, String info) {
        if (isTraceOn()) {
            trace(dbgTag, func, info);
        }
    }
    
    static boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    static void debug(String clz, String func, String info) {
        if (isDebugOn()) {
            Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
        }
    }

    static void debug(String clz, String func, Throwable exception) {
        if (isDebugOn()) {
            Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
        }
    }

    static void debug(String func, String info) {
        if (isDebugOn()) {
            debug(dbgTag, func, info);
        }
    }
    
    static void debug(String func, Throwable exception) {
        if (isDebugOn()) {
            debug(dbgTag, func, exception);
        }
    }
    
    static String dbgTag = "SnmpV3AppliMibRegistration"; 
}

