package com.sun.management.snmp.rfc2573.notification.mib;

// java imports
//
import java.io.Serializable;
import java.net.UnknownHostException;

// jmx imports
//
import com.sun.management.snmp.SnmpStatusException;

// jmx imports
//
import com.sun.management.snmp.SnmpStatusException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.MalformedObjectNameException;
// jdmk imports
//
import com.sun.jdmk.trace.Trace;
import com.sun.jdmk.ServiceName;
import com.sun.management.snmp.agent.SnmpMib;
import com.sun.management.snmp.EnumRowStatus;

//Contrib imports
import com.sun.management.snmp.rfc2573.notification.SnmpNotificationLcd;
import com.sun.management.snmp.rfc2573.notification.SnmpNotificationData;

public class SnmpNotifyEntryImpl extends SnmpNotifyEntry {

    SnmpNotificationLcd lcd = null;
    SnmpNotificationData data = null;

    public SnmpNotifyEntryImpl(SnmpMib theMib, 
			       SnmpNotificationLcd lcd) {
	super(theMib);
	this.lcd = lcd;
    }

    //Constructor called when instanciating from conf.
    public SnmpNotifyEntryImpl(SnmpMib theMib, 
				   SnmpNotificationLcd lcd, 
				   SnmpNotificationData data) throws IllegalArgumentException, UnknownHostException {
	super(theMib);
	this.lcd = lcd;
	this.data = data;
	
	SnmpNotifyName = data.notifyName;
	byte[] b = data.notifyTag.getBytes();
	int size = b.length;
	SnmpNotifyTag = new Byte[size];
	for(int i = 0; i < size; i++)
	    SnmpNotifyTag[i] = new Byte(b[i]);

	SnmpNotifyType = new EnumSnmpNotifyType(data.notifyType);
	SnmpNotifyStorageType  = new EnumSnmpNotifyStorageType(data.notifyStorage);
    }

    /**
     * Checker for the "SnmpTargetAddrRowStatus" variable.
     */
    public void setSnmpNotifyRowStatus(EnumSnmpNotifyRowStatus x) throws SnmpStatusException {

	if(x.intValue() == EnumRowStatus.active) {
	    try {


		//Make the tags.
		int size = SnmpNotifyTag.length;
		byte[] tags = new byte[size];
		for(int i = 0; i < size; i++) 
		    tags[i] = SnmpNotifyTag[i].byteValue();

		lcd.addNotification(SnmpNotifyName, 
				    new String(tags), 
				    SnmpNotifyType.intValue(), 
				    SnmpNotifyStorageType.intValue());
		super.setSnmpNotifyRowStatus(x);

	    }catch(IllegalArgumentException e) {
		
		throw new SnmpStatusException(e.getMessage());
	    }
	}
    }

    public void addInMBeanServer(MBeanServer server) {
	try {
	    //Use the prefix.
            server.registerMBean(this, createObjectName());
        } catch (Exception e) {
        }
    }

    protected ObjectName createObjectName() 
	throws MalformedObjectNameException {
	return new ObjectName("TableSnmpNotifyTable:name=com.sun.management.snmp.rfc2573.notification.mib.SnmpNotifyEntry" + ",SnmpNotifyName=" + SnmpNotifyName);
    }

    
}
