/*
 * @(#)file      TableSnmpTargetParamsTableImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.3
 * @(#)date      02/01/29
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.target.mib;

import java.util.Enumeration;

import com.sun.jdmk.trace.Trace;

// jmx imports
//
import javax.management.MBeanServer;
import javax.management.ObjectName;
import com.sun.management.snmp.SnmpCounter;
import com.sun.management.snmp.SnmpCounter64;
import com.sun.management.snmp.SnmpGauge;
import com.sun.management.snmp.SnmpInt;
import com.sun.management.snmp.SnmpUnsignedInt;
import com.sun.management.snmp.SnmpIpAddress;
import com.sun.management.snmp.SnmpTimeticks;
import com.sun.management.snmp.SnmpOpaque;
import com.sun.management.snmp.SnmpString;
import com.sun.management.snmp.SnmpStringFixed;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpNull;
import com.sun.management.snmp.SnmpValue;
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpStatusException;

// jdmk imports
//
import com.sun.management.snmp.usm.SnmpUsmMibTable;
import com.sun.management.snmp.usm.SnmpUsmLcd;
import com.sun.management.snmp.usm.SnmpUsmSecureUser;
import com.sun.management.snmp.usm.SnmpUsmAlgorithm;
import com.sun.management.snmp.agent.SnmpIndex;
import com.sun.management.snmp.agent.SnmpMib;
import com.sun.management.snmp.agent.SnmpMibTable;
import com.sun.management.snmp.agent.SnmpMibSubRequest;
import com.sun.management.snmp.agent.SnmpTableEntryFactory;
import com.sun.management.snmp.agent.SnmpTableCallbackHandler;
import com.sun.management.snmp.agent.SnmpTableSupport;
import com.sun.management.snmp.EnumRowStatus;

//Contrib imports
import com.sun.management.snmp.rfc2573.target.SnmpTargetLcd;
import com.sun.management.snmp.rfc2573.target.SnmpParamsData;

class TableSnmpTargetParamsTableImpl extends TableSnmpTargetParamsTable {
    boolean init = false;
    SnmpTargetLcd lcd = null;

    public TableSnmpTargetParamsTableImpl(SnmpMib myMib, 
					  SnmpTargetLcd lcd) {
	super(myMib);
	init(lcd);
	init = true;
    }
    
    public TableSnmpTargetParamsTableImpl(SnmpMib myMib,
					  MBeanServer server,
					  SnmpTargetLcd lcd) {
	super(myMib, server);
	init(lcd);
	init = true;
    }
    
    //For debugging purpose only.
    public void addEntryCb(int pos, SnmpOid row, ObjectName name, 
			   Object entry, SnmpMibTable meta) 
	throws SnmpStatusException {
	if(isDebugOn())
	    debug("addEntryCb","");
    }
    
    /**
     * A user has been removed from SNMP.
     */
    public void removeEntryCb(int pos, SnmpOid row, ObjectName name, 
			      Object entry, SnmpMibTable meta)
	throws SnmpStatusException {
	super.removeEntryCb(pos, row, name, entry, meta);
	if(isDebugOn())
	    debug("removeEntryCb","");
	//Ask the Lcd to remove The user from the conf.
	SnmpTargetParamsEntry e = (SnmpTargetParamsEntry) entry;
	lcd.removeParams(e.SnmpTargetParamsName);
    }

    public Object createSnmpTargetParamsEntryMBean(SnmpMibSubRequest req,
						   SnmpOid rowOid, 
						   int depth, 
						   ObjectName entryObjName,
						   SnmpMibTable meta, 
						   String  aSnmpTargetParamsName) throws SnmpStatusException  {
        // Note that when using standard metadata,
        // the returned object must implement the "SnmpTargetParamsEntryMBean"
        // interface.
        //
        SnmpTargetParamsEntryImpl entry = new SnmpTargetParamsEntryImpl(theMib,
									lcd);
        entry.SnmpTargetParamsName = aSnmpTargetParamsName;
        return entry;
    }
    
    private void init(SnmpTargetLcd lcd) {
	setCreationEnabled(true);
	this.lcd = lcd;
	
	//Access the all set of users.
	Enumeration params = lcd.getParams();
	while(params.hasMoreElements()) {
	    SnmpParamsData data = (SnmpParamsData) params.nextElement();
	    SnmpTargetParamsEntryImpl entry = 
		new SnmpTargetParamsEntryImpl(theMib, 
					      lcd, 
					      data);
	    entry.SnmpTargetParamsRowStatus = new EnumSnmpTargetParamsRowStatus(EnumRowStatus.active);
	    if(isDebugOn()) {
		debug("init","Adding :" + data.paramsName);
	    }
	    
	    try {
		//Add the entry in the table.
		addEntry(entry);
		if(isDebugOn()) {
		    debug("init","Added :" + data.paramsName);
		}
		
		if(server != null)
		    entry.addInMBeanServer(server);
		
	    }catch(SnmpStatusException s) {
		if(isDebugOn())
		    debug("init", s);
	    }
	}
    }
    
    // TRACES & DEBUG
    //---------------
    
    boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
    }

    void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
    }

    void debug(String clz, String func, Throwable exception) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
    }

    void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
    
    void debug(String func, Throwable exception) {
        debug(dbgTag, func, exception);
    }
    
    String dbgTag = "TableSnmpTargetParamsTableImpl"; 
}
