/*
 * @(#)file      SnmpTargetAddrEntryImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.2
 * @(#)date      01/12/20
 *
 * Copyright 2000 Sun Microsystems, Inc. All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000 Sun Microsystems, Inc. Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
package com.sun.management.snmp.rfc2573.v3proxy.mib;


// java imports
//
import java.io.Serializable;
import java.net.UnknownHostException;
import java.net.InetAddress;

// jmx imports
//
import com.sun.management.snmp.SnmpStatusException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.MalformedObjectNameException;
import com.sun.management.snmp.SnmpEngineId;

// jdmk imports
//
import com.sun.jdmk.trace.Trace;
import com.sun.jdmk.ServiceName;
import com.sun.management.snmp.agent.SnmpMib;
import com.sun.management.snmp.EnumRowStatus;

//Contrib imports
import com.sun.management.snmp.rfc2573.target.SnmpTargetLcd;
import com.sun.management.snmp.rfc2573.v3proxy.SnmpProxyLcd;
import com.sun.management.snmp.rfc2573.v3proxy.SnmpProxyData;

public class SnmpProxyEntryImpl extends SnmpProxyEntry {
    //Constructor called from the SNMP MIB.
    SnmpTargetLcd targetLcd = null;
    SnmpProxyLcd proxyLcd = null;
    SnmpProxyData data = null;
    public SnmpProxyEntryImpl(SnmpMib theMib, 
			      SnmpProxyLcd proxyLcd,
			      SnmpTargetLcd targetLcd) {
	super(theMib);
	this.targetLcd = targetLcd;
	this.proxyLcd = proxyLcd;
    }
    
    //Constructor called when instanciating from conf.
    public SnmpProxyEntryImpl(SnmpMib theMib, 
			      SnmpProxyLcd proxyLcd,
			      SnmpTargetLcd targetLcd, 
			      SnmpProxyData data) throws IllegalArgumentException, UnknownHostException {
	super(theMib);
	this.targetLcd = targetLcd;
	this.proxyLcd = proxyLcd;
	this.data = data;
	
	SnmpProxyName = data.proxyName;
	SnmpProxySingleTargetOut = data.singleOut.targetName;
	SnmpProxyTargetParamsIn = data.paramsIn.paramsName;
	SnmpProxyContextName = data.contextName;

	byte[] multiple = data.multipleOut.getBytes();
	int size = multiple.length;
	SnmpProxyMultipleTargetOut = new Byte[size];
	for(int i = 0; i < size; i++)
	    SnmpProxyMultipleTargetOut[i] = new Byte(multiple[i]);
	
	byte[] ctxEngineId = data.contextEngineId.getBytes();
	size = ctxEngineId.length;
	SnmpProxyContextEngineID = new Byte[size];
	for(int i = 0; i < size; i++)
	    SnmpProxyContextEngineID[i] = new Byte(ctxEngineId[i]);
	
	SnmpProxyType = new EnumSnmpProxyType(data.type);
	
	SnmpProxyStorageType = new EnumSnmpProxyStorageType(data.storageType);
    }
    
    /**
     * Checker for the "SnmpTargetAddrRowStatus" variable.
     */
    public void setSnmpProxyRowStatus(EnumSnmpProxyRowStatus x) throws SnmpStatusException {
	//Do it here or NOT.
	if(isDebugOn())
	    debug("setSnmpProxyRowStatus", "Received row status is : " + x.intValue());
	if(x.intValue() == EnumRowStatus.active) {
	    try {
		
		//Make the tags.
		int size = SnmpProxyMultipleTargetOut.length;
		byte[] multi = new byte[size];
		
		for(int i = 0; i < size; i++) 
		    multi[i] = SnmpProxyMultipleTargetOut[i].byteValue();
		
		if(isDebugOn())
		    debug("setSnmpProxyRowStatus", "multiple done");
		size = SnmpProxyContextEngineID.length;
		byte[] ctx = new byte[size];

		
		for(int i = 0; i < size; i++)
		    ctx[i] = SnmpProxyContextEngineID[i].byteValue();
		
		SnmpEngineId id = SnmpEngineId.createEngineId(ctx);
		if(isDebugOn())
		    debug("setSnmpProxyRowStatus", "ctxtEngineId done : " +
			  id);
		if(isDebugOn())
		    debug("setSnmpProxyRowStatus", "Adding proxy");
		proxyLcd.addProxy(SnmpProxyName,
				  SnmpProxyType.intValue(),
				  id,
				  SnmpProxyContextName,
				  SnmpProxyTargetParamsIn,
				  SnmpProxySingleTargetOut,
				  new String(multi),
				  SnmpProxyStorageType.intValue());
		if(isDebugOn())
		    debug("setSnmpProxyRowStatus", "Proxy added");
		if(isDebugOn())
		    debug("setSnmpProxyRowStatus",
			  "Row status active. addTarget done.");
		super.setSnmpProxyRowStatus(x);
	    }catch(IllegalArgumentException e) {
		if(isDebugOn())
		    debug("setSnmpProxyRowStatus", e);
		throw new SnmpStatusException(e.getMessage());
	    }
	}
    }

    public void addInMBeanServer(MBeanServer server) {
	try {
	    //Use the prefix.
            server.registerMBean(this, createObjectName());
        } catch (Exception e) {
	    if(isTraceOn())
		trace("addInMBeanServer", e.toString());
        }
    }
    
    protected ObjectName createObjectName() 
	throws MalformedObjectNameException {
	return new ObjectName("TableSnmpProxyTable:name=com.sun.management.snmp.v3proxy.mib.SnmpProxyEntry" + ",SnmpProxyName=" + SnmpProxyName);
    }
    
    // TRACES & DEBUG
    //---------------
    
    boolean isTraceOn() {
        return Trace.isSelected(Trace.LEVEL_TRACE, Trace.INFO_SNMP);
    }

    void trace(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_TRACE, Trace.INFO_SNMP, clz, func, info);
    }

    void trace(String func, String info) {
        trace(dbgTag, func, info);
    }
    
    boolean isDebugOn() {
        return Trace.isSelected(Trace.LEVEL_DEBUG, Trace.INFO_SNMP);
    }

    void debug(String clz, String func, String info) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, info);
    }

    void debug(String clz, String func, Throwable exception) {
        Trace.send(Trace.LEVEL_DEBUG, Trace.INFO_SNMP, clz, func, exception);
    }

    void debug(String func, String info) {
        debug(dbgTag, func, info);
    }
    
    void debug(String func, Throwable exception) {
        debug(dbgTag, func, exception);
    }
    
    String dbgTag = "SnmpProxyEntryImpl"; 
}
