/*
 * @(#)file      SyncV3Manager.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.14
 * @(#)date      04/07/05
 *
 * Copyright 2000-2004 Sun Microsystems, Inc.  All rights reserved.
 * This software is the proprietary information of Sun Microsystems, Inc.
 * Use is subject to license terms.
 * 
 * Copyright 2000-2004 Sun Microsystems, Inc.  Tous droits r?serv?s.
 * Ce logiciel est propriet? de Sun Microsystems, Inc.
 * Distribu? par des licences qui en restreignent l'utilisation. 
 *
 */
// java import
//
import java.net.SocketException;

// RI import
//
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpVarBindList;
import com.sun.management.snmp.SnmpStatusException;
import com.sun.management.snmp.manager.usm.SnmpUsmPeer;
import com.sun.management.snmp.manager.SnmpPeer;
import com.sun.management.snmp.manager.SnmpParameters;
import com.sun.management.snmp.manager.SnmpRequest;
import com.sun.management.snmp.manager.SnmpSession;
import com.sun.management.snmp.manager.SnmpEventReportDispatcher;
import com.sun.management.snmp.manager.usm.SnmpUsmParameters;
import com.sun.management.snmp.SnmpEngineId;
import com.sun.management.snmp.SnmpEngine;
import com.sun.management.snmp.SnmpEngineParameters;

// jdmk import
//
import com.sun.jdmk.TraceManager;
import com.sun.jdmk.trace.Trace;
import com.sun.management.snmp.SnmpOidTableSupport;
import com.sun.jdmk.comm.HtmlAdaptorServer;
import com.sun.management.comm.SnmpV3AdaptorServer;
import com.sun.management.comm.SnmpV3InformRequest;
import com.sun.management.comm.SnmpInformRequest;

// RI import
//
import com.sun.management.snmp.SnmpPduRequest;
import com.sun.management.snmp.SnmpPduTrap;
import com.sun.management.snmp.manager.SnmpTrapListener;
import com.sun.management.snmp.manager.SnmpInformListener;
import com.sun.management.snmp.SnmpScopedPduRequest;
import com.sun.management.snmp.SnmpEngineId;

// Contrib import
import com.sun.management.snmp.rfc2573.manager.SnmpV3AppliMibRegistration;


/**
 * The class is a simple example showing how to use the SNMP Manager API.
 *
 * When calling the program, you must specify:
 *      - host: hostname of the SNMP agent you want to query.
 *      - port: port number to use.
 *
 * As a reminder, if you wish to query the SNMP agent example provided
 * as part of JDMK, use port 8085.
 *
 * In order to get traces from the SNMP Manager API, invoke the program as follows:
 *      java -DINFO_SNMP SyncManager <hostname> 8085
 */

class TrapListenerImpl implements SnmpTrapListener {

    public void processSnmpTrapV1(SnmpPduTrap trap) {
        println("NOTE: TrapListenerImpl received trap :");
        println("\tGeneric " + trap.genericTrap);
        println("\tSpecific " + trap.specificTrap);
        println("\tTimeStamp " + trap.timeStamp);
        println("\tAgent adress " + trap.agentAddr.stringValue());
    }

    public void processSnmpTrapV2(SnmpPduRequest trap) {
        println("NOTE: Trap V2 not of interest !!!");
    }

    public void processSnmpTrapV3(SnmpScopedPduRequest trap) {
	println("NOTE: TrapListenerImpl received trap V3:");
	println("\tContextEngineId : " + 
		SnmpEngineId.createEngineId(trap.contextEngineId));
	println("\tContextName : " + new String(trap.contextName));
	println("\tVarBind list :");
	for(int i = 0; i < trap.varBindList.length; i++)
	    println("oid : " + trap.varBindList[i].getOid() + " val : " + 
		    trap.varBindList[i].getSnmpValue());
	
    }

    /**
     * print/println stuff...
     **/
    private final static void println(String msg) {
	java.lang.System.out.println(msg);
    }
    private final static void print(String msg) {
	java.lang.System.out.print(msg);
    }

}

class  InformListenerImpl implements SnmpInformListener {
   
    public void processSnmpInform(SnmpPduRequest inform) {
        
        // Display the received PDU.
        //
        println("\nNOTE: Inform request received:\n");
        println("\tType      = " + inform.pduTypeToString(inform.type) + "\n");
        println("\tVersion   = " + inform.version + "\n");
        println("\tRequestId = " + inform.requestId + "\n");
        println("\tAddress   = " + inform.address + "\n");
        println("\tPort      = " + inform.port + "\n");
        println("\tCommunity = " + new String(inform.community) + "\n");
        println("\tVB list   = ");
        for (int i = 0; i < inform.varBindList.length; i++) {
            println("\t\t" + inform.varBindList[i] + "\n");
        }
    }
    
    public void processSnmpInformV3(SnmpScopedPduRequest inform) {
	println("\nNOTE: Inform request received:\n");
	println("\tContextEngineId : " + new String(inform.contextEngineId) + "\n");
	println("\tContextName : " + new String(inform.contextName) + "\n");
	println("\tMsgFlags : " + inform.msgFlags + "\n");
	println("\tMsgMaxSize : " + inform.msgMaxSize + "\n");
	println("\tMsgSecurityModel : " + inform.msgSecurityModel + "\n");
	println("\tAgent list :"+"\n");
	for(int i = 0; i < inform.varBindList.length; i++)
	  println("oid : " + 
			   inform.varBindList[i].getOid() + " val : " + 
			   inform.varBindList[i].getSnmpValue() + "\n");

	println("**************INFORM OVER******************\n");
    }
      
    public void processSnmpInternalError(SnmpInformRequest request, 
					 String errmsg) {
        
        println("\nAgentRspHandler::processSnmpInternalError: " + 
		request.toString());
        println("Error is: " + errmsg);
    }

    /**
     * print/println stuff...
     **/
    private final static void println(String msg) {
	java.lang.System.out.println(msg);
    }
    private final static void print(String msg) {
	java.lang.System.out.print(msg);
    }
}

public class SyncV3Manager {
   
    /**
     * Main entry point.
     * When calling the program, you must specify:
     *  1) host: hostname of the SNMP agent you want to query.
     *  2) port: port number to use.
     *
     * As a reminder, if you wish to query the SNMP agent example provided
     * as part of JDMK, use port 8085.
     *
     * In order to get traces from the SNMP Manager API, invoke the program as follows:
     *  java -DINFO_SNMP SyncManager <hostname> 8085
     */
    public static void main(String argv[]) {
    
	// Default value for the peer node
        String host = "localhost";
        int port = 8085;
	int trapPort = 8084;

        // Initialize trace property. If you want to get traces while using the
        // SNMP Manager API, invoke the program with "-DINFO_SNMP".
        // 
        try {
            TraceManager.parseTraceProperties();
            Trace.send(Trace.LEVEL_TRACE, Trace.INFO_MISC, "SyncV3Manager", "main", "Default trace is on");
        } catch(java.io.IOException e) {
            e.printStackTrace();
        }
  
        try {
            
            // Initialize the SNMP Manager API.
            // Specify the OidTable containing all the MIB II knowledge.
            // Use the OidTable generated by mibgen when compiling MIB II.
            //
            SnmpOidTableSupport oidTable = new RFC1213_MIBOidTable();
            SnmpOid.setSnmpOidTable(oidTable);
       
            // At any time, it is possible to add knowledge in the OidTable
            // using the SnmpOidDatabaseSupport.
            // For instance, we could start the Manager API like that:
            //    SnmpOidDatabaseSupport oidDB = new SnmpOidDatabaseSupport();
            //    SnmpOid.setSnmpOidTable(oidDB);
            // Then over time, MIB knowledge can be added as follows:
            //    oidDB.add(oidTable);
            //
            
            // Create a SnmpPeer object for representing the entity to communicate with. 
            //

	    //Create the session
	    SnmpSession session = null;
	    SnmpEngineId id = null;
	    try {
		SnmpEngineParameters p = new SnmpEngineParameters();
		p.setEngineId(SnmpEngineId.createEngineId("localhost:8083"));
		session= new SnmpSession(p,
					 null,
					 "SyncV3Manager session",
					 null);
		java.lang.System.out.println("LOCAL ENGINE Id (localhost:8083) :" + id);
	    }catch(SnmpStatusException e) {
		
	    }
	    
	    //We compute the engine Id using the Inform port and host.
	    //This is for ex purpose only. Can be set using conf file or var env.
	    //This could become systematic. When setting an Inform Dispatcher ==> Set the engineId to host:port.
	    
	    
	    //Create the Usm Peer.
            SnmpUsmPeer agent = new SnmpUsmPeer(session.getEngine(),
						host, 
						port);
	    java.lang.System.out.println("agent.getEngineId()" + agent.getEngineId());
	    //Create Parameters objects
	    //The agent will use the updated params boot and time
	    SnmpUsmParameters p = new SnmpUsmParameters(session.getEngine(), 
							"defaultUser");
	    p.setContextEngineId(agent.getEngineId().getBytes());
	    p.setSecurityLevel(SnmpDefinitions.authNoPriv);
	    agent.setParams(p);
	    
	    //Will make some authenticated requests, must discover timeliness infos
	    agent.processUsmTimelinessDiscovery();
	    
	    java.lang.System.out.println("agent.getEngineBoots()" + 
					 agent.getEngineBoots());
	    java.lang.System.out.println("agent.getEngineTime()" +
					 agent.getEngineTime());
            
	    
            // A default peer (agent) can be associated to a SnmpSession. When invoking a service
            // provided by the SnmpSession, if the agent is not specified, the session will perform
            // the service using the default peer as the target of the service.
            //
            session.setDefaultPeer(agent);

	    try {

		java.lang.System.out.println("Press a Key to register the target to receive V1 Trap/Inform");
		java.lang.System.in.read();

		// Params V1
		SnmpV3AppliMibRegistration.registerParams(session,
							  "manager_paramsv1",
							  0, 
							  0,
							  "public",
							  1);
		
		// To receive V1 trap use Params V1
		SnmpV3AppliMibRegistration.registerTarget(session,
							  "manager_targetv1",
							  "1.3.6.1.6.1.1",
							  "localhost/"+trapPort,
							  10000,
							  2,
							  "notify",
							  "manager_paramsv1");

		SnmpV3AppliMibRegistration.registerForTrap(session,
							   "manager_trapv1",
							   "notify");
		
		java.lang.System.out.println("Press a Key to register the target to receive V3 Trap/Inform");
		java.lang.System.in.read();

		SnmpV3AppliMibRegistration.registerParams(session,
							  "manager_paramsv3", 
							  3, 
							  3,
							  "defaultUser",
							  SnmpDefinitions.authNoPriv);
		
		SnmpV3AppliMibRegistration.registerTarget(session,
							  "manager_targetv3",
							  "1.3.6.1.6.1.1",
							  "localhost/"+trapPort,
							  10000,
							  2,
							  "notify",
							  "manager_paramsv3");
				     
	
	    }catch(SnmpStatusException e) {
		System.out.println("ERROR" + e.getMessage());
	    }

	    // Create the Trap listener
	    TrapListenerImpl trapListener = new TrapListenerImpl();
	    InformListenerImpl informListener = new InformListenerImpl();
	    SnmpEventReportDispatcher trapAgent = null;
	    try{
		trapAgent = new SnmpEventReportDispatcher(8084);
	    } catch(SocketException e) { 
		e.printStackTrace(); 
	    }
	    new Thread(trapAgent).start();
	    
	    trapAgent.addTrapListener(trapListener); 
	    trapAgent.addInformListener(informListener);
 
            // Build the list of variables you want to query.
            // For debug purposes, you can associate a name to your list.
            //

	    SnmpUsmParameters params = new SnmpUsmParameters(session.getEngine(), 
							     "defaultUser");
	    params.setContextName("TEST-CONTEXT".getBytes());
	    params.setContextEngineId(agent.getEngineId().getBytes());
	    params.setSecurityLevel(SnmpDefinitions.authNoPriv);
	    agent.setParams(params);

            SnmpVarBindList list = new SnmpVarBindList("SyncManager varbind list");
	    
            // We want to read the "sysDescr" variable.
            //
            list.addVarBind("sysDescr.0");
	    boolean flags = true;
	    while(true) {
		try {
		    
		    if(flags) {
			java.lang.System.out.println("Press a Key to send a get on SNMP V1 agent");
			params.setContextEngineId(SnmpEngineId.createEngineId("localhost:8095").getBytes());
			flags = false;
		    } else {
			java.lang.System.out.println("Press a Key to send a get on SNMP V3 agent");
			params.setContextEngineId(SnmpEngineId.createEngineId("localhost:8105").getBytes());
			flags = true;
		    }
		    java.lang.System.in.read();
		    if(sendGet(session, agent, list, host, port) == -1) {
			java.lang.System.err.println("SyncManager::main: Error when sending get.");
		    }
		    
		}catch(SnmpStatusException e){
		    e.printStackTrace();
		}
	    }
	    
        } catch(Exception e) {
            java.lang.System.err.println("SyncManager::main: Exception occurred:" + e );
            e.printStackTrace();
        }
    }
   
    /**
     * Return usage of the program.
     */
    static private int sendGet(SnmpSession session,
			       SnmpUsmPeer agent,
			       SnmpVarBindList list,
			       String host,
			       int port) throws Exception {
	// Make the SNMP get request and wait for the result.
	//
	SnmpRequest request = session.snmpGetRequest(null, list);
	java.lang.System.out.println("SyncManager::main: Send get request to SNMP agent on " + host + " at port " + port);
	boolean completed = request.waitForCompletion(10000);
	
	// Check for a timeout of the request.
	//
	if (completed == false) {
	    java.lang.System.out.println("SyncManager::main: Request timed out. Check reachability of agent");
	    
	    // Print request.
	    //
	    java.lang.System.out.println("Request: " + request.toString());
	    return -1;
	}
       
	// Now we have a response. Check if the response contains an error.
	//
	int errorStatus = request.getErrorStatus();
	if (errorStatus != SnmpDefinitions.snmpRspNoError) {
	    java.lang.System.out.println("Error status = " + SnmpRequest.snmpErrorToString(errorStatus));
	    java.lang.System.out.println("Error index = " + request.getErrorIndex());
	    return -1;
	}
	SnmpVarBindList result = request.getResponseVarBindList();
	if( request.isReport() ) {
	    java.lang.System.out.println("REPORT RECEIVED, SHOULD RETRY");
	    java.lang.System.out.println(request.getReportVarBindList());
	    return -1;
	}
	else
	    java.lang.System.out.println("Result: \n" + result);
	return 0;
	
    }
    public static void  usage() {
        java.lang.System.out.println("java -Djdmk.security.file=./jdmk.security SyncV3Manager <host> <port>");
        java.lang.System.out.println("where");
        java.lang.System.out.println("\t-host: hostname or IP address of the SNMP agent you want to query.");
        java.lang.System.out.println("\t-port: port number to use.");
    }
}
