/*
 * @(#)file      AgentRspHandler.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.5
 * @(#)lastedit  04/04/07
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// java import
//
import java.util.Vector;
import java.util.Enumeration;

// RI import
//
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpVarBindList;

// jdmk import
//
import com.sun.management.comm.SnmpInformHandler;
import com.sun.management.comm.SnmpInformRequest;


/**
 * This class is a simple implementation of an inform response handler.
 * The class is used by the SNMP adaptor server for handling responses.
 */

public class AgentRspHandler implements SnmpInformHandler {
   
    /**
     * Create a handler.
     */
    public AgentRspHandler() {
    }
   
    public void processSnmpPollData(SnmpInformRequest request, 
				    int errStatus, int errIndex, 
				    SnmpVarBindList vblist) {
     
        println("\nAgentRspHandler::processSnmpPollData: " + 
		request.toString());
        println("errStatus = " + 
		SnmpInformRequest.snmpErrorToString(errStatus) + 
		" errIndex = " + errIndex);
        
        // Check if a result is available.
        //
        if (request.getRequestStatus() == 
	    SnmpInformRequest.stResultsAvailable) {
            // A result is available. Go and get it !
            //
            SnmpVarBindList result = request.getResponseVarBindList();
            println("Result = " + result.varBindListToString());
        }
    }
   
    public void processSnmpPollTimeout(SnmpInformRequest request) {
        
        println("\nAgentRspHandler::processSnmpPollTimeout: " + 
		request.toString());
        
        if (request.getRequestStatus() == 
	    SnmpInformRequest.stResultsAvailable) {
            // A result is available. Go and get it !
            //
            SnmpVarBindList result = request.getResponseVarBindList();
            println("Result = " + result.varBindListToString());
        }
    }
      
    public void processSnmpInternalError(SnmpInformRequest request, 
					 String errmsg) {
        
        println("\nAgentRspHandler::processSnmpInternalError: " + 
		request.toString());
        println("Error is: " + errmsg);
    }

    /**
     * print/println stuff...
     **/
    private final static void println(String msg) {
	java.lang.System.out.println(msg);
    }
    private final static void print(String msg) {
	java.lang.System.out.print(msg);
    }

}
