/*
 * @(#)file      JmxMBeanServerImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.7
 * @(#)lastedit  04/05/11
 * 
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// java imports
//
import java.io.Serializable;
import java.util.Set;
import java.util.Map;
import java.util.List;
import java.util.HashMap;
import java.util.Iterator;

// jmx/jdmk imports
//
import javax.management.MBeanServer;
import com.sun.management.snmp.SnmpString;
import com.sun.management.snmp.SnmpInt;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpVarBind;
import com.sun.management.snmp.SnmpStatusException;
import javax.management.ObjectName;
import javax.management.MBeanServerNotification;
import javax.management.MBeanServerConnection;
import javax.management.Notification;
import javax.management.MBeanNotificationInfo;
import javax.management.NotificationEmitter;
import javax.management.NotificationListener;
import javax.management.NotificationFilter;
import javax.management.NotificationBroadcasterSupport;
import javax.management.InstanceAlreadyExistsException;
import javax.management.InstanceNotFoundException;
import javax.management.MalformedObjectNameException;
import com.sun.management.snmp.agent.SnmpMib;
import com.sun.management.snmp.agent.SnmpMibTable;
import com.sun.management.snmp.agent.SnmpMibSubRequest;
import com.sun.management.snmp.EnumRowStatus;


/**
 * This class is used for instrumenting the "JmxMBeanServer" group.
 * It extends the mibgen generated JmxMBeanServer skeleton.
 * 
 *<p> This class instantiate a subclass of the generated TableJmxMBeanTable
 * that mirrors the MBeans registered in an MBeanServer. It listen for
 * MBeanServerNotification in order to keep the jmxMBeanTable up-to-date.
 * It also creates/deletes the associated rows in the jmxMBeanAttrTable.
 *
 */
public class JmxMBeanServerImpl extends JmxMBeanServer {
    
    // The MIB to which this object belongs.
    //
    final JMX_MBEAN_SERVER_MIB_Impl myMib;

    /**
     * Constructor for the "JmxMBeanServer" group.
     */
    public JmxMBeanServerImpl(SnmpMib myMib) {
	super(myMib);
	this.myMib = (JMX_MBEAN_SERVER_MIB_Impl)myMib;
	inittables();
    }


    /**
     * Constructor for the "JmxMBeanServer" group.
     * 
     */
    public JmxMBeanServerImpl(SnmpMib myMib, MBeanServer server) {
	// We do not want to have the mibgen generated MBeans
	// registered in the MBeanServer: this would cause
	// an infinite loop, since the registered MBean would be
	// automatically mirrored in the MIB, hence triggering the
	// registration of a new generated entry MBean etc....
	// 
	// We thus call the super constructor without passing the 
	// MBeanServer.
	//
	// Note: we could still register the generated MBeans in an
	//       MBeanServer ptovided that it is a different MBeanServer
	//       that returned by getMibServer(). 
	//
	super(myMib);
	this.myMib = (JMX_MBEAN_SERVER_MIB_Impl)myMib;
	inittables();
    }

    private final void inittables() {
	// We do not want to have the mibgen generated MBeans
	// registered in the MBeanServer: this would cause
	// an infinite loop, since the registered MBean would be
	// automatically mirrored in the MIB, hence triggering the
	// registration of a new generated entry MBean etc....
	// 
	// We thus call the constructor with a 'null' MBeanServer.
	//
	// We need to override the createJmxMBeanEntryMBean only because
	// we want to allow remote entry creation in the JmxMBeanTable.
	//
	JmxMBeanTable = new TableJmxMBeanTable(myMib,null) {
		// Will return an JmxMBeanEntryImpl object, instead of
		// the generated JmxMBeanEntry skeleton.
		//
		public Object createJmxMBeanEntryMBean(SnmpMibSubRequest req,
						       SnmpOid rowOid, 
						       int depth, 
						       ObjectName entryObjName,
						       SnmpMibTable meta, 
						       Integer  aJmxMBeanIndex)
		    throws SnmpStatusException  {
		    
		    return JmxMBeanServerImpl.this.
			createJmxMBeanEntryMBean(req,rowOid,depth,entryObjName,
						 meta,aJmxMBeanIndex);
		}

		// We override this method so that it also make the entry
		// delete its attributes in the jmxMBeanAttrTable.
		// This is done by calling e.setJmxMBeanRowStatus(destroy);
		//
		public void removeEntryCb(int pos, SnmpOid row, 
					  ObjectName name,
					  Object entry, SnmpMibTable meta)
		    throws SnmpStatusException {
		    super.removeEntryCb(pos,row,name,entry,meta);
		    final JmxMBeanEntryMBean e = 
			(JmxMBeanEntryMBean) entry;
		    final EnumJmxMBeanRowStatus destroy =
			new EnumJmxMBeanRowStatus(EnumRowStatus.destroy);
		    e.setJmxMBeanRowStatus(destroy);
		}
	    };
	
	// Allow remote entry creation in the jmxMBeanTable.
	//
	JmxMBeanTable.setCreationEnabled(true);
    }


    /**
     * The MBeanServer that is mirrored by this MIB. 
     * Note: the MIB is not expected to be registered in an MBeanServer.
     *       It is only registered in the SnmpAdaptorServer.
     **/
    public MBeanServer getMibServer() {
	return  myMib.getMibServer();
    }


    /**
     * Getter for the "JmxMBeanCount" variable.
     */
    public Integer getJmxMBeanCount() throws SnmpStatusException {
	try {
	    return myMib.getMibServer().getMBeanCount();
	} catch (Exception x) {
            throw new SnmpStatusException(SnmpStatusException.snmpRspGenErr);
	}
    }

    // ----------------------------------------------------------------------
    //
    // Initialization: when this object is started it populates the 
    //                 jmxMBeanTable with the content of the MBeanServer
    //                 and starts listening for MBeanServerNotifications.
    //
    // ----------------------------------------------------------------------
    boolean started = false;

    /**
     * Starts monitoring the MBeanServer.
     * When this object is started it populates the jmxMBeanTable with the 
     * content of the MBeanServer and starts listening for 
     * MBeanServerNotifications.
     **/
    public synchronized void start() {

	started = true;
	try {

	    // Starts listening for MBeanServerNotifications
	    //
	    final ObjectName delegate = 
		new ObjectName("JMImplementation:type=MBeanServerDelegate");
	    myMib.getMibServer().
		addNotificationListener(delegate,mbsListener,null,null);

	    try {
		// Populates the jmxMBeanTable and jmxMBeanAttr table with
		// the initial content of the MBeanServer.
		//
		Set mbeans = myMib.getMibServer().queryNames(null,null);
		for (final Iterator it = mbeans.iterator(); it.hasNext();) {
		    final ObjectName name = (ObjectName) it.next();
		    if (mustShow(name)) showMBean("start",name);
		}
	    } catch (Exception x) {
		try {
		    myMib.getMibServer().
			removeNotificationListener(delegate,mbsListener);
		} catch (Exception e) { /* OK */ }
		throw x;
	    }
	} catch (Exception x) {
	    started = false;	    
	    System.err.println("Failed to start MBean Table: " + x);
	    if (Boolean.getBoolean("debug")) x.printStackTrace();
	}
    }


    //------------------------------------------------------------------------
    // 
    // Management of the jmxMBeanAttrTable table.
    //
    //------------------------------------------------------------------------

    /**
     * Populate the jmxMBeanAttrTable with the attributes of the
     * given JmxMBeanEntryImpl entry.
     * This method is called when the given entry is activated (that is,
     * when its RowStatus is switched to "active".
     * @param name  The ObjectName of the MBean mirrored by the given entry.
     * @param entry The entry in the jmxMBeanAttrTable that mirrors the MBean
     *              named by <var>name</var>.
     **/
    synchronized int addAttributes(ObjectName name, JmxMBeanEntryImpl entry) 
	throws SnmpStatusException {
	int added = 0;
	final List attrs = entry.getAttributeList();
	try {
	    for (int i=0; i<attrs.size();i++) {
		final JmxMBeanAttrEntry e = (JmxMBeanAttrEntry)(attrs.get(i));
		JmxMBeanAttrTable.addEntry(e);
		added++;
	    }
	} catch (Exception x) {
	    System.err.println("Failed to add all attributes for: " + name);
	    if (Boolean.getBoolean("debug")) x.printStackTrace();
	}
	if (Boolean.getBoolean("info")) 
	    System.out.println("Added " + added + "/" + attrs.size()
			       + " attribute(s).");
	return added;
    }

    /**
     * Populate the jmxMBeanAttrTable with the attributes of the
     * JmxMBeanEntryImpl entry.
     * This method is called when the given entry is activated (that is,
     * when its RowStatus is switched to "destroy".
     * @param name  The ObjectName of the MBean mirrored by the given entry.
     * @param entry The entry in the jmxMBeanAttrTable that mirrors the MBean
     *              named by <var>name</var>.
     **/
    synchronized int removeAttributes(ObjectName name, 
				       JmxMBeanEntryImpl entry) 
	throws SnmpStatusException {
	final int exposed = entry.getExposedAttrCount();
	if (Boolean.getBoolean("info")) 
	    System.out.println(name.toString()+ ": must remove " + exposed
			       + " attribute(s).");

	final List attrs = entry.getAttributeList();
	int removed = 0;
	for (int i=0; i<attrs.size();i++) {
	    final JmxMBeanAttrEntry e = (JmxMBeanAttrEntry)(attrs.get(i));
	    try {
		JmxMBeanAttrTable.removeEntry(e);
		removed++;
	    } catch (Exception x) {
		System.err.println("Failed to remove attribute["+i+"]"
				   +" for: " + name);
		if (Boolean.getBoolean("debug")) x.printStackTrace();
	    }
	}
	if (Boolean.getBoolean("info")) 
	    System.out.println("Removed " + removed + "/" + exposed 
			       + " attribute(s).");
	return removed;	
    }

    //------------------------------------------------------------------------
    // 
    // Management of the jmxMBeanAttrTable table.
    //
    //------------------------------------------------------------------------

    /** 
     * This method is called from JmxMBeanEntryImpl, when a remote SNMP
     * manager trigger the creation of a new MBean through a SET request.
     * @param mbean The just created MBean to be registered.
     * @param name  The ObjectName of the given MBean entry.
     * @param entry The entry in the jmxMBeanAttrTable that mirrors the 
     *              new MBean.
     **/
    synchronized void registerMBean(Object mbean, ObjectName name, 
				    JmxMBeanEntry entry) throws Exception {
	names.put(name,entry);
	try {
	    myMib.getMibServer().registerMBean(mbean,name);
	} catch (Exception x) {
	    names.remove(name);
	    throw x;
	}
    }

    /** 
     * This method is called from JmxMBeanEntryImpl, when a remote SNMP
     * manager trigger the removal of a register MBean through a SET request.
     * @param name  The ObjectName of the MBean mirrored by the given entry.
     * @param entry The entry in the jmxMBeanAttrTable that mirrors the MBean
     *              named by <var>name</var>.
     **/
    synchronized void unregisterMBean(ObjectName name, 
				      JmxMBeanEntry entry) 
	throws Exception {
	names.remove(name);
	try {
	    myMib.getMibServer().unregisterMBean(name);
	} catch (Exception x) {
	    names.put(name,entry);
	    throw x;
	}
    }

    /**
     * This method is called when a remote SNMP Manager creates a new
     * row in the jmxMBeanTable.
     * This method is called from the overridden createJmxMBeanEntryMBean(...)
     * method of TableJmxMBeanTable - see inittables().
     * @param req The SnmpMibSubRequest containing the VarBind list
     *            pertaining to this new row.
     * @param rowOid The index part of the OID for this new row.
     * @param depth  The depth at which the arc of the columnar objects
     *               will be found in the OID part of the VarBind.
     * @param entryObjName The ObjectName that would be used to register
     *        this entry in an MBeanServer - as it stands, this MIB does
     *        not register its own MBeans into the MBeanServer as it could
     *        lead to infinite loop (each MBean being mirrored by an entry
     *        a new entry would be needed to mirror the new MBean etc...)
     *        Therefore, this parameter is ignored.
     *        <p><b>Note:</b> Do not confuse the instance of this entry
     *        [an instance of the generated JmxMBeanEntryMBean class], which
     *        this method returns, but that we are not going to register
     *        anywhere but in the jmxMBeanAttrTable, with the MBean mirrored 
     *        by this entry [an instance of whatever class was specified in 
     *        the SET request sent by the manager], which we are going to
     *        instantiate and eventually register in the MBeanServer.
     * @param meta The SnmpMibTable metadata object of the jmxMBeanTable.
     * @param aJmxMBeanIndex The value of the JmxMBeanIndex index variable
     *        for the new entry.
     **/
    private Object createJmxMBeanEntryMBean(SnmpMibSubRequest req,
					    SnmpOid rowOid, 
					    int depth, 
					    ObjectName entryObjName,
					    SnmpMibTable meta, 
					    Integer  aJmxMBeanIndex)
	throws SnmpStatusException  {

	
	SnmpVarBind rowStatusVb = req.getRowStatusVarBind();

	if (rowStatusVb == null) 
	    throw new SnmpStatusException(
		      SnmpStatusException.snmpRspNoCreation);

	// Check whether the provided index can be used.
	//
	if (! isAvailable(aJmxMBeanIndex.intValue())) {
	    if (Boolean.getBoolean("info"))
		System.out.println("Index is not suitable: " + aJmxMBeanIndex);
	    throw new SnmpStatusException(
		      SnmpStatusException.snmpRspInconsistentValue);
	}

	// Create the new entry
	//
	JmxMBeanEntryImpl entry = 
	    new JmxMBeanEntryImpl(myMib,this);
	entry.JmxMBeanIndex = aJmxMBeanIndex;

	// Perform the necessary initialization step that are needed
	// when a row is created remotely:
	//   * Local creation of row is triggered by the reception of
	//     a MBeanServerNotification: the MBean it mirrors is 
	//     already registered
	//   * Remote creation of a row is triggered by an SNMP SET request,
	//     and will trigger the creation of a new MBean - whose ObjectName
	//     and Class Name are supplied in the SET request. Creating
	//     and registering this MBean will in turn trigger an 
	//     MBeanServerNotification.
	// This means that special care must be taken so that local/remote
	// creation do not step on each other.
	//
	// The call to createFromRemote() will
	//     1) check the validity of the input parameter,
	//     2) store all the needed information so that the requested
	//        MBean is finally created and registered.
	//     3) ensure that the MBeanServerNotification emitted as a result
	//        of this registration does not create a dupplicated row
	//        at a different index. 
	// 
	entry.createFromRemote(req,rowStatusVb);
	return entry;
    }

    //------------------------------------------------------------------------
    // 
    // Management of MBeanServerNotifications. 
    //
    // When an MBeanServerNotification is received, the jmxMBeanTable  
    // and jmxMBeanAttrTable are updated accordingly, by creating/deleting
    // the rows that mirror the create/deleted MBean
    //
    //------------------------------------------------------------------------

    /**
     * This method is called by <var>mbsListener</var> - see start() - when
     * an MBeanServerNotification is received.
     **/
    private void handleMBSNotification(Notification notif, Object handback) {
	// Make sure we do nothing until we're started.
	//
	synchronized(this) { if (!started) return; }

	// Make sure we're handling the right type of Notification 
	//
	if (notif instanceof MBeanServerNotification) {
	    final MBeanServerNotification n = 
		(MBeanServerNotification) notif;
	    final String nt = n.getType();
	    final ObjectName mbeanName = n.getMBeanName();
	    
	    if (MBeanServerNotification.REGISTRATION_NOTIFICATION.
		equals(nt)) {

		// A new MBean was registered in the MBeanServer: add it to
		// the tables!
		//
		synchronized (this) {
		    if (mustShow(mbeanName)) { 
			showMBean(nt,mbeanName);
		    }
		} 
	    } else if (MBeanServerNotification.UNREGISTRATION_NOTIFICATION.
		       equals(nt)) {

		// A registered MBean was removed from the MBeanServer: 
		// remove it from the tables!
		//
		synchronized (this) {
		    if (mustHide(mbeanName)) {
			hideMBean(nt,mbeanName);
		    }
		}
	    }
	}

    }

    /**
     * Make sure we need to add this MBean in the tables (if it's been
     * created from a SET request, it's already there).
     **/
    private boolean mustShow(ObjectName name) {
	return !names.containsKey(name);
    }
    
    /**
     * Make sure we need to remove this MBean from the tables (if it's been
     * destroyed by a SET request, it has been already removed).
     **/
    private boolean mustHide(ObjectName name) {
	return names.containsKey(name);
    }
    
    /**
     * Shows an MBean through the jmxMBeanTable and jmxMBeanAttrTables.
     **/
    private void showMBean(String operation,ObjectName name) {
	try {
	    final int index = freeindex();
	    updatelast(index);
	    showMBean(operation,name,index);
	} catch (Exception x) {
	    System.err.println("Failed to add MBean entry: " + name);
	    if (Boolean.getBoolean("debug")) x.printStackTrace();
	}
    }

    /**
     * Shows an MBean through the jmxMBeanTable and jmxMBeanAttrTables.
     **/
    private void showMBean(String operation,ObjectName name,int index) {
	try {
	    JmxMBeanEntry entry = new JmxMBeanEntryImpl(myMib,this);
	    entry.JmxMBeanIndex = new Integer(index);
	    entry.setJmxMBeanObjectName(name.toString());
	    JmxMBeanTable.addEntry(entry);
	    names.put(name, entry);
	    entry.setJmxMBeanRowStatus(
		  new EnumJmxMBeanRowStatus(EnumRowStatus.active));
	    if (Boolean.getBoolean("info")) 
		System.out.println("ADDED: JmxMBeanTable["+index+"]="+name);
	} catch (Exception x) {
	    System.err.println("Failed to add MBean entry: " + name);
	    if (Boolean.getBoolean("debug")) x.printStackTrace();
	}
    }

    /**
     * Remove an MBean from the jmxMBeanTable and jmxMBeanAttrTables.
     **/
    private void hideMBean(String operation,ObjectName name) {
	try {
	    final JmxMBeanEntryImpl entry = 
		(JmxMBeanEntryImpl) names.get(name);
	    try {
		entry.removeFromRemote(false);
		JmxMBeanTable.removeEntry(entry);
		if (Boolean.getBoolean("info")) 
		    System.out.println("REMOVED: JmxMBeanTable["+
				       entry.JmxMBeanIndex+"]="+name);
	    } finally {
		names.remove(names);
	    }
	} catch (Exception x) {
	    System.err.println("Failed to remove MBean entry: " + name);
	    if (Boolean.getBoolean("debug")) x.printStackTrace();
	}
    }

    // ---------------------------------------------------------------------
    //
    // Management of indexes:
    // 
    // The isAvailable() and updatelast() method are required only because
    // we allow remote entry creation in the jmxMBeanTable.
    // If the table were read-only (updated by the agent only) things would
    // be much simpler.
    //
    // ---------------------------------------------------------------------
    //
    // Last reserved index. Remote manager must provide an index value
    // greater than this variable. This is not a very satisfactory
    // behavior - but it keeps things simple. We could have devised
    // a kind of "spinlock" mechanism, as in the SNMPv3 framework MIBs,
    // but that would have been an overkill for that example.
    //
    private int last = 0;

    /**
     * This method is used to "reserve" an index for the jmxMBeanTable.
     **/
    private final synchronized void updatelast(int index) {
	if (last < index) last = index;
    }

    /**
     * Check whether the given index can be used.
     * This method is called when an SNMP Manager attempts to create
     * a row in the jmxMBeanTable. The index will be accepted only if
     * it is greater than the last allocated index, and if it 
     * is greater than the greatest index used in the table.
     **/
    private final synchronized boolean isAvailable(int index) {
	if ((last < index) &&  (index >= freeindex())) { 
	    last = index; return true; 
	}
	return false;
    }

    /**
     * This method is used to find a suitable index when adding a new
     * row in the jmxMBeanTable upon reception of an MBeanServerNotification.
     * 
     * Look for the index of the last element in the table. 
     * This will be the greatest index. 
     * Returns that index + 1.
     **/
    private final synchronized int freeindex() {
	final int size = JmxMBeanTable.getSize();
	if (size == 0) return 1;
	final JmxMBeanEntry entry = (JmxMBeanEntry) 
	    JmxMBeanTable.getEntry(size-1);
	if (entry == null) return last+1;
	return entry.JmxMBeanIndex.intValue() + 1;
    }


    // The mbsListener is registered with the MBeanServer delegate in
    // order to receive MBeanServerNotifications - see start().
    //
    final NotificationListener mbsListener = new NotificationListener() {
	    public void handleNotification(Notification n, Object handback) {
		handleMBSNotification(n,handback);
	    }
	};

    // This map is used to make sure that local creation - triggered
    // by MBeanServerNotifications and remote creation - triggered by
    // SNMP SET Request do not step on each-other's toes.
    //
    final Map names = new HashMap(); 


}
