/*
 * @(#)file      TrapListenerImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.9
 * @(#)lastedit  04/04/07
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// JDMK imports
//
import com.sun.management.snmp.SnmpPduRequest;
import com.sun.management.snmp.SnmpPduTrap;
import com.sun.management.snmp.SnmpScopedPduRequest;
import com.sun.management.snmp.SnmpEngineId;
import com.sun.management.snmp.manager.SnmpTrapListener;

/**
 * This class implements the SnmpTrapListener interface.
 * The callback method processSnmpTrapV1 is called when
 * a valid trap PDU is received.
 */

public class TrapListenerImpl implements SnmpTrapListener {

    public void processSnmpTrapV1(SnmpPduTrap trap) {
        println("NOTE: TrapListenerImpl received SNMP V1 trap:");
        println("\tGeneric " + trap.genericTrap);
        println("\tSpecific " + trap.specificTrap);
        println("\tTimeStamp " + trap.timeStamp);
        println("\tAgent address " + trap.agentAddr.stringValue());
    }

    public void processSnmpTrapV2(SnmpPduRequest trap) {
        println("NOTE: TrapListenerImpl received SNMP V2 trap:");
        println("\tCommunityString : " + new String(trap.community));
        println("\tVarBind list :");
        for (int i = 0; i < trap.varBindList.length; i++)
            println("oid : " + trap.varBindList[i].getOid() + " val : " +
                    trap.varBindList[i].getSnmpValue());
    }

    public void processSnmpTrapV3(SnmpScopedPduRequest trap) {
        println("NOTE: TrapListenerImpl received trap V3:");
        println("\tContextEngineId : " + 
                SnmpEngineId.createEngineId(trap.contextEngineId));
        println("\tContextName : " + new String(trap.contextName));
        println("\tVarBind list :");
        for (int i = 0; i < trap.varBindList.length; i++)
            println("oid : " + trap.varBindList[i].getOid() + " val : " +
                    trap.varBindList[i].getSnmpValue());
    }

    /**
     * print/println stuff...
     */

    private final static void println(String msg) {
        java.lang.System.out.println(msg);
    }

    private final static void print(String msg) {
        java.lang.System.out.print(msg);
    }
}
