/*
 * @(#)file      SystemImpl.java
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.4
 * @(#)lastedit  04/04/07
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

// java import
//
import java.net.InetAddress;
import java.net.UnknownHostException;

// RI import
//
import javax.management.MBeanServer;
import com.sun.management.snmp.SnmpStatusException;

// jdmk import
//
import com.sun.management.snmp.agent.SnmpMib;


/**
 * The SystemImpl class provides a simple implementation of the "System"
 * group, showing how to retrieve the community string from the UserData
 * object previously allocated by the SnmpUserDataFactory defined in the
 * Agent class.
 *
 */

public class SystemImpl extends System {

    // MBean properties.
    //
    
    // Start up time of the agent.
    //
    private long startUpTime = 0;

    /**
     * Constructors.
     */
    public SystemImpl(SnmpMib myMib) {
    
        super(myMib);
        init();
    }
    
    public SystemImpl(SnmpMib myMib, MBeanServer server) {
    
        super(myMib, server);
        init();
    }
    
    private void init() {
        
        // Initialize the system description using some system properties.
        //
        try {
            SysDescr = java.lang.System.getProperty("os.name") + " " +
                java.lang.System.getProperty("os.arch") + " " +
                java.lang.System.getProperty("os.version");
        } catch(SecurityException e) {
            // Do not process the exception
        }
    
        // Initialize the system name using the hostname.
        //
        try {
            SysName = (InetAddress.getLocalHost()).getHostName();
        } catch(UnknownHostException e) {
            // Do not process the exception
        }
    
        // Initialize the system contact using some system properties.
        //
        try {
            SysContact = java.lang.System.getProperty("user.name");
        } catch(SecurityException e) {
            // Do not process the exception
        }
    
        // Initialize the set of services. Assume the agent only deals 
	// with application layer.
        //
        SysServices = new Integer(72);

        // Initialize the location with a dummy string.
        //
        SysLocation = new String("Sample implementation of system group.");
    
        // For the sysUpTime, use the time the agent started...
        //
        startUpTime = java.lang.System.currentTimeMillis();
    }

    // ----------------------------------------------------------------------
    //
    // Redefine the Getters/Checkers inherited from the generated System
    // stub in order to print the manager supplied community string.
    //
    // ----------------------------------------------------------------------


    public Long getSysUpTime() throws SnmpStatusException {
	// Print the community string on stdout
	//
	printCommunity("GET sysUpTime");
        long diff = java.lang.System.currentTimeMillis() - startUpTime;
        return new Long(diff);
    }

    public Integer getSysServices() throws SnmpStatusException {
	// Print the community string on stdout
	//
	printCommunity("GET sysServices");
        return super.getSysServices();
    }

    public String getSysLocation() throws SnmpStatusException {
	// Print the community string on stdout
	//
	printCommunity("GET sysServices");
        return super.getSysLocation();
    }

    public void checkSysLocation(String x) throws SnmpStatusException {
	// Print the community string on stdout
	//
	printCommunity("SET sysLocation");
    }

    public String getSysName() throws SnmpStatusException {
	// Print the community string on stdout
	//
	printCommunity("GET sysName");
        return super.getSysName();
    }

    public void checkSysName(String x) throws SnmpStatusException {
	// Print the community string on stdout
	//
	printCommunity("SET sysName");
    }

    public String getSysContact() throws SnmpStatusException {
	// Print the community string on stdout
	//
	printCommunity("GET sysContact");
        return super.getSysContact();
    }

    public void checkSysContact(String x) throws SnmpStatusException {
	// Print the community string on stdout
	//
	printCommunity("SET sysContact");
    }

    public String getSysObjectID() throws SnmpStatusException {
	// Print the community string on stdout
	//
	printCommunity("GET sysObjectId");
        return super.getSysObjectID();
    }

    public String getSysDescr() throws SnmpStatusException {
	// Print the community string on stdout
	//
	printCommunity("GET sysDescr");
        return super.getSysDescr();
    }

    /**
     * Print the manager supplied community string on stdout.
     *
     **/
    private void printCommunity(String primitive) {

	// Get the manager supplied community string from the UserData
	// object associated with the current thread context.
	//
	String community = Agent.getManagerCommunity();

	// Print the community string on stdout
	//
	if (community == null) {
	    java.lang.System.out.println(primitive + 
					 ": Community String = <NONE>");
	} else {
	    java.lang.System.out.println(primitive + 
					 ": Community String = \"" + 
					 community + "\"" );
	}
    }
    
}
