/*
 * @(#)file      KernelStatImpl.c
 * @(#)author    Sun Microsystems, Inc.
 * @(#)version   1.4
 * @(#)lastedit  04/02/16
 *
 * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * Kstat native API, JNI lib.
 */

#include <jni.h>
#include <com_sun_jdmk_contrib_kstat_KernelStat.h>
#include <stdio.h>
#include <kstat.h>
#include <strings.h>

#define LEVEL_DEBUG 0x02
#define LEVEL_ERROR 0x01
#ifndef TRACE_MASK
#define TRACE_MASK  0x01
#endif
#define TRACE(tag,level,...) ((level&TRACE_MASK)?(fprintf((level&LEVEL_ERROR?stderr:stdout),"%s ",tag),fprintf((level&LEVEL_ERROR?stderr:stdout),__VA_ARGS__),0):0)

#define BUFLEN 10240

/* Iterate the data records and store name;value pairs in buffer */
int fillData(kstat_ctl_t* kc, kstat_t* ksp, char* buf, int *found) {
    kstat_named_t *kn;
    int i = 0;
    char str[512];

    kn = (kstat_named_t *)ksp->ks_data;

    /* for each record */
    for (i = 0; i < ksp->ks_ndata; i++) {
        char knn[KSTAT_STRLEN+1];
    knn[0]='\0';
    if(kn->name == NULL) {
        kn++;
        continue;
    }
    strncpy(knn,kn->name,KSTAT_STRLEN);
    knn[KSTAT_STRLEN]='\0';
    *found = 1;
    /* According to the type, store the name and value */
    switch (kn->data_type) {
    case KSTAT_DATA_CHAR:
        sprintf(str, "%s;%ld.%ld.%ld.%ld;", knn, (ulong)kn->value.c,
            (ulong)(kn->value.c+4),(ulong)(kn->value.c+8),
            (ulong)(kn->value.c+12));
        break;
    case KSTAT_DATA_INT32:
        sprintf(str, "%s;%ld;", knn, kn->value.i32);
        break;
    case KSTAT_DATA_UINT32:
        sprintf(str, "%s;%ld;", knn, kn->value.ui32);
        break;
    case KSTAT_DATA_INT64:
        sprintf(str, "%s;%lld;", knn, kn->value.i64);
        break;
    case KSTAT_DATA_UINT64:
        sprintf(str, "%s;%lld;", knn, kn->value.ui64);
        break;
    default:
        TRACE("KSTAT(JNI)", LEVEL_ERROR, "Invalid union data type %d for property %s\n", kn->data_type, knn);
        continue;
    }

    /* If the buffer is too small, return its current value. */
    if (strlen(buf) > (BUFLEN - strlen(str))) {
        TRACE("KSTAT(JNI)", LEVEL_DEBUG, "Too many kstat properties, Buffer not big enough. Truncate\n");
        /*kstat_close(kc);*/
        return 0;
    }
    /* add to the list of properties */
    strcat(buf, str);
    
    /* inc the record pointer */
    kn++;
    } 
    return 1;
}

void closeAndFree(JNIEnv  *env,
          kstat_ctl_t   *kc,
          const char* mName,
          const char* iName,
          const char* pName,
          jstring modName,
          jstring instanceName,
          jstring propName) {
    if (modName != NULL)
    (*env)->ReleaseStringUTFChars(env, modName, mName);
    
    if (instanceName != NULL)
    (*env)->ReleaseStringUTFChars(env, instanceName, iName);
    
    if (propName != NULL)
    (*env)->ReleaseStringUTFChars(env, propName, pName);
    
    kstat_close(kc);
}

void closeAndFreeModAndInstAndProp(JNIEnv  *env,
                   kstat_ctl_t   *kc,
                   const char* mName,
                   const char* iName,
                   const char* pName,
                   jstring modName,
                   jstring instanceName,
                   jstring propName) {
    closeAndFree(env, 
         kc, 
         mName, 
         iName, 
         pName, 
         modName, 
         instanceName, 
         propName);
}

void closeAndFreeModAndProp(JNIEnv  *env,
                kstat_ctl_t   *kc,
                const char* mName,
                const char* pName,
                jstring modName,
                jstring propName) {
    closeAndFree(env, kc, mName, NULL, pName, modName, NULL, propName);
}

void closeAndFreeModAndInst(JNIEnv  *env,
                kstat_ctl_t   *kc,
                const char* mName,
                const char* iName,
                jstring modName,
                jstring instanceName) {
    closeAndFree(env, kc, mName, iName, NULL, modName, instanceName, NULL);
}

void closeAndFreeMod(JNIEnv  *env,
             kstat_ctl_t   *kc,
             const char* mName,
             jstring modName) {
    closeAndFree(env, kc, mName, NULL, NULL, modName, NULL, NULL);
}

/* Store the passed property value in the buffer */
int fillProperty(const char *pName, kstat_ctl_t* kc, kstat_t* ksp, char* buf, int *found) {
    kstat_named_t *kn;
    char          str[512];
    int           i =0;
    /*
     * kstat read
     */
    if (kstat_read(kc, ksp, NULL) < 0) {
    TRACE("KSTAT(JNI)", LEVEL_ERROR, 
               "kstat_read error : \n");
    /*kstat_close(kc);*/
    return 0;
    }
    
    /* Access data */
    kn = (kstat_named_t *)ksp->ks_data;
    
    for (i = 0; i < ksp->ks_ndata; i++) {
        char knn[KSTAT_STRLEN+1];
    knn[0]='\0';
    if(kn->name == NULL) {
        kn++;
        continue;
    }

    strncpy(knn,kn->name,KSTAT_STRLEN);
    knn[KSTAT_STRLEN]='\0';

    if (pName != NULL) {
        if (strcmp(pName, knn) != 0) {
        kn++;
        continue;
        }
    }
    
    *found = 1;
    
    switch (kn->data_type) {
    case KSTAT_DATA_CHAR:
        sprintf(str, "%ld.%ld.%ld.%ld", (ulong)kn->value.c,
            (ulong)(kn->value.c+4),(ulong)(kn->value.c+8),
            (ulong)(kn->value.c+12));
        break;
    case KSTAT_DATA_INT32:
        sprintf(str, "%ld", kn->value.i32);
        break;
    case KSTAT_DATA_UINT32:
        sprintf(str, "%ld", kn->value.ui32);
        break;
    case KSTAT_DATA_INT64:
        sprintf(str, "%lld", kn->value.i64);
        break;
    case KSTAT_DATA_UINT64:
        sprintf(str, "%lld", kn->value.ui64);
        break; 
    default:
        TRACE("KSTAT(JNI)", LEVEL_ERROR, "Invalid union data type %d for property %s\n", kn->data_type, knn);
    }
    
    if (buf[0] != 0)
        strcat(buf, " ");
    if (strlen(buf) > (BUFLEN - strlen(str))) {
        TRACE("KSTAT(JNI)", LEVEL_ERROR, "Too many kstat properties, Buffer not big enough. Truncate\n");
        /*kstat_close(kc);*/
        return 0;
    }
    
    /* store the val in buffer */
    strcat(buf, str);
    
    /* Exit the properties loop */
    break;
    }

    return 1;
}

/*
 * Class:     KernelStat
 * Method:    init
 * Does some init, traces, ...
 */
JNIEXPORT void JNICALL 
Java_com_sun_jdmk_contrib_kstat_KernelStat_init(
                      JNIEnv  *env,
                      jobject obj)
{
  TRACE("KSTAT(JNI)", LEVEL_DEBUG, "initialized\n");\
}

/*
 * Class:     KernelStat
 * Method:    statValues
 * The list of property name/value for a kstat element.
 * return a string with:
 * <property name>;<property val>;...
 */
JNIEXPORT jstring JNICALL 
    Java_com_sun_jdmk_contrib_kstat_KernelStat_statValues(
                            JNIEnv  *env,
                            jobject obj,
                            jstring modName,
                            jint instance,
                            jstring instanceName)
{
    const char    *mName;
    const char    *iName;
    long          ktype;
    kstat_ctl_t   *kc;
    kstat_t       *ksp;
    char          buf[BUFLEN];
    int           found = 0;

    buf[0] = 0;

    TRACE("KSTAT(JNI)", LEVEL_DEBUG, "statAllValueList\n");

    /*
     * kstat open
     */ 
    if ((kc = kstat_open()) == NULL) {
      TRACE("KSTAT(JNI)", LEVEL_ERROR, "kstat_open error : \n");
      return NULL;
    }
    
    if (modName == NULL) {
      mName = NULL;
    }
    else {
      mName = (*env)->GetStringUTFChars(env, modName, 0);
    }

    if (instanceName == NULL) {
      iName = NULL;
    }
    else {
      iName = (*env)->GetStringUTFChars(env, instanceName, 0);
    }

  
    /*
     * Process the lookup
     */
    if ((ksp = kstat_lookup(kc, (char*)mName, instance, (char *)iName)) == NULL) {
    TRACE("KSTAT(JNI)", LEVEL_ERROR, "kstat_lookup error : \n");
    closeAndFreeModAndInst(env,
                   kc,
                   mName,
                   iName,
                   modName,
                   instanceName);
    return NULL;
    }  
  
    /*
     * types are !=, return NULL
     */
    if (ksp->ks_type != KSTAT_TYPE_NAMED) {
    closeAndFreeModAndInst(env,
                   kc,
                   mName,
                   iName,
                   modName,
                   instanceName);
      return NULL;
    }
    /*
     * Process the read.
     */
    if (kstat_read(kc, ksp, NULL) < 0) {
    TRACE("KSTAT(JNI)", LEVEL_ERROR, "kstat_read error : \n");
    closeAndFreeModAndInst(env,
                   kc,
                   mName,
                   iName,
                   modName,
                   instanceName);
    return NULL;
    }
    
    /* Store the properties */    
    if(!fillData(kc, ksp, buf, &found)) {
    closeAndFreeModAndInst(env,
                   kc,
                   mName,
                   iName,
                   modName,
                   instanceName); 
      return (*env)->NewStringUTF(env, buf);
    }

    closeAndFreeModAndInst(env,
               kc,
               mName,
               iName,
               modName,
               instanceName);
    
    if (found) {
    return ((*env)->NewStringUTF(env, buf));
    }
    else {
    return NULL;
    }
}

/*
 * Class:     KernelStat
 * Method:    statAllValueList
 *
 * For a given module returns returns the list of properties. Each kstat element list of properties is separated by a white space.
 * return a string with:
 * <property11 name>;<property11 val>;... <property21 name>;<property21 val>;...
 */
JNIEXPORT jstring JNICALL 
Java_com_sun_jdmk_contrib_kstat_KernelStat_statValuesList(
                            JNIEnv  *env,
                            jobject obj, 
                            jstring modName)
{
    const char    *mName;
    const char    *pName;
    long          ktype;
    kstat_ctl_t   *kc;
    kstat_t       *ksp;
    kstat_named_t *kn;
    char          buf[BUFLEN];
    char          str[512];
    int           found = 0, i =0;
    
    /* Buffer boot strap */
    buf[0] = 0;

    TRACE("KSTAT(JNI)", LEVEL_DEBUG, "statValuesList\n");
    /*
     * kstat open
     */ 
    if ((kc = kstat_open()) == NULL) {
      TRACE("KSTAT(JNI)", LEVEL_ERROR, "kstat_open error : \n");    
      return NULL;
    }
    
    if (modName == NULL) {
      mName = NULL;
    }
    else {
      mName = (*env)->GetStringUTFChars(env, modName, 0);
    }

    /*
     * Iterates on the whole kstat chained list.
     */
    for (ksp = kc->kc_chain; ksp != NULL; ksp = ksp->ks_next) {
      char knn[KSTAT_STRLEN+1];
      char knm[KSTAT_STRLEN+1];
      knn[0]=knm[0]='\0';

      /* Get module name */ 
      strncpy(knm,ksp->ks_module,KSTAT_STRLEN);
      knm[KSTAT_STRLEN]='\0';


      if (mName != NULL) {
    /*
     * module names are !=, continue.
     */
    if (strcmp(mName, knm) != 0) {
      continue;
    }
      }
      /*
       * types are !=, continue.
       */
      if (ksp->ks_type != KSTAT_TYPE_NAMED) {
      continue;
      }
      
      /*
       * kstat read
       */
      if (kstat_read(kc, ksp, NULL) < 0) {
      TRACE("KSTAT(JNI)", LEVEL_ERROR, "kstat_read error : \n");
      closeAndFreeMod(env,
              kc,
              mName,
              modName);
      return NULL;
      }

      /* Get instance name */ 
      strncpy(knn,ksp->ks_name,KSTAT_STRLEN);
      knn[KSTAT_STRLEN]='\0';

      /* A new kstat, add the name at the start of the property list */
      sprintf(buf, "%s;", knn);

      /* Store the properties */    
      if(!fillData(kc, ksp, buf, &found)) {
      closeAndFreeMod(env,
              kc,
              mName,
              modName);
    return (*env)->NewStringUTF(env, buf);
      }
      
      /* Add a white space separator.*/       
      if (buf[0] != 0)
    strcat(buf, " ");
    }
    
    /*
     * kstat close
     */
    /*kstat_close(kc);*/
    closeAndFreeMod(env,
            kc,
            mName,
            modName);
    
    if (found) {
      return ((*env)->NewStringUTF(env, buf));
    }
    else {
      return NULL;
    }
}
/*
 * Class:     KernelStat
 * Method:    statValueList
 * Go through all kstat element and stores the values of the passed property name
 * return a string with:
 *   <kstat1 val>;<kstat2 val>;...
 */
JNIEXPORT jstring JNICALL 
    Java_com_sun_jdmk_contrib_kstat_KernelStat_statValueList(
                               JNIEnv  *env,
                               jobject obj, 
                               jstring modName,
                               jstring propertyName)
{
    const char    *mName;
    const char    *pName;
    long          ktype;
    kstat_ctl_t   *kc;
    kstat_t       *ksp;
    kstat_named_t *kn;
    char          buf[BUFLEN];
    int           found = 0;
    
    /* Buffer boot strap */
    buf[0] = 0;

    TRACE("KSTAT(JNI)", LEVEL_DEBUG, "statValueList\n");

    /*
     * kstat open
     */ 
    if ((kc = kstat_open()) == NULL) {
      TRACE("KSTAT(JNI)", LEVEL_ERROR, "kstat_open error : \n");
      return NULL;
    }
    
    if (modName == NULL) {
      mName = NULL;
    }
    else {
      mName = (*env)->GetStringUTFChars(env, modName, 0);
    }
    if (propertyName == NULL) {
      pName = NULL;
    }
    else {
      pName = (*env)->GetStringUTFChars(env, propertyName, 0);
    }
   
    /*
     * Iterates on the whole kstat chained list.
     */
    for (ksp = kc->kc_chain; ksp != NULL; ksp = ksp->ks_next) {
      
      if (mName != NULL) {
    /*
     * module names are !=, continue.
     */
    if (strcmp(mName, ksp->ks_module) != 0) {
      continue;
    }
      }
      /*
       * types are !=, continue.
       */
      if (ksp->ks_type != KSTAT_TYPE_NAMED) {
    continue;
      }
      
      /* Store the property */
      if(!fillProperty(pName, kc, ksp, buf, &found)) {
      closeAndFreeModAndProp(env,
                 kc,
                 mName,
                 pName,
                 modName,
                 propertyName); 
    return (*env)->NewStringUTF(env, buf);
      }
    }

    /*
     * kstat close
     */
    closeAndFreeModAndProp(env,
               kc,
               mName,
               pName,
               modName,
               propertyName); 
    
    if (found) {
      return ((*env)->NewStringUTF(env, buf));
    }
    else {
      return NULL;
    }
}

/*
 * Class:     KernelStat
 * Method:    statGetList
 * Signature: (Ljava/lang/String;ILjava/lang/String;I)Ljava/lang/String;
 *
 * Look for all <module name> <instance name> for a kstat type
 * return a string with:
 *   <module name>:<instnace val>:<name>
 */
JNIEXPORT jstring JNICALL 
Java_com_sun_jdmk_contrib_kstat_KernelStat_statGetList(
                JNIEnv  *env,
                jobject obj, 
                jstring modName,
                jint    instance,
                jstring instName, 
                jint    type)
{
  const char    *mName;
  const char    *iName;
  long          ktype;
  kstat_ctl_t   *kc;
  kstat_t       *ksp;
  char          buf[BUFLEN];
  char          str[512];
  int           found = 0;

  /* Buffer boot strap */
  buf[0] = 0;
  
  TRACE("KSTAT(JNI)", LEVEL_DEBUG, "statGetList\n");

  /*
   * kstat open
   */ 
  if ((kc = kstat_open()) == NULL) {
    TRACE("KSTAT(JNI)", LEVEL_ERROR, "kstat_open error : \n");
    return NULL;
  }
  
  if (modName == NULL) {
    mName = NULL;
  }
  else {
    mName = (*env)->GetStringUTFChars(env, modName, 0);
  }

  if (instName == NULL) {
    iName = NULL;
  }
  else {
    iName = (*env)->GetStringUTFChars(env, instName, 0);
  }

  ktype = type;
  

  /*
   * Iterates on the whole kstat chained list.
   */
  for (ksp = kc->kc_chain; ksp != NULL; ksp = ksp->ks_next) {

    /* This is a hack to make sure that the name are \0 terminated. */
    char knm[KSTAT_STRLEN+1];
    char knn[KSTAT_STRLEN+1];
    knn[0]=knm[0]='\0';

    /* Get module name */
    strncpy(knm,ksp->ks_module,KSTAT_STRLEN);
    knm[KSTAT_STRLEN]='\0';

    /* Get instance name */
    strncpy(knn,ksp->ks_name,KSTAT_STRLEN);
    knn[KSTAT_STRLEN]='\0';

    
    if (mName != NULL) {
      /*
       * module names are !=, continue.
       */

      if (strcmp(mName, knm) != 0) {
    continue;
      }
    }

    if (iName != NULL) {
      /*
       * names are !=, continue.
       */

      if (strcmp(iName, knn) != 0) {
    continue;
      }
    }

    if (instance != -1) {
      /* 
       * instance number are !=, continue.
       */
      if ((int)instance != ksp->ks_instance) {
        continue;
      }
    }

    if (ktype != -1) {
      /*
       * types are !=, continue.
       */
      if (ksp->ks_type != (int)ktype) {
    continue;
      }
    }

    found = 1;
    /*
     * Tokens are " " separated
     */
    if (buf[0] != 0) {
      strcat(buf, " ");
    }
    /*
     * Add it to the result buff.
     */
    sprintf(str, "%s;%d;%s", knm, ksp->ks_instance, knn);

    TRACE("KSTAT(JNI)", LEVEL_DEBUG, "kstat found : %s\n", str);
    if (strlen(buf) > (BUFLEN - strlen(str))) {
      TRACE("KSTAT(JNI)", LEVEL_DEBUG, 
        "Too many kstat properties, Buffer not big enough. Truncate\n");
      /*kstat_close(kc);*/
      closeAndFreeModAndInst(env,
                 kc,
                 mName,
                 iName,
                 modName,
                 instName);
      return ((*env)->NewStringUTF(env, buf));
    }
    /* cat it to the end of buf).*/
    strcat(buf, str);
  }
  
  /*
   * kstat close
   */
  closeAndFreeModAndInst(env,
             kc,
             mName,
             iName,
             modName,
             instName);

  if (found) {
    return ((*env)->NewStringUTF(env, buf));
  }
  else {
      /*
       * THIS TRACE BREAKS THE GENERICITY OF KSTAT API...
       * SHOULD BE REMOVED AFTER C/JAVA trace merging.
       */
      TRACE("KSTAT(JNI)", LEVEL_ERROR, 
        "No kernel statistics available\n");
      return NULL;
  }
}


  /*
   * Class:     KernelStat
   * Method:    statGetValue
   * Signature: (Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;
   * Returns the kstat property value associated to the passed arguments.
   */
JNIEXPORT jstring JNICALL
Java_com_sun_jdmk_contrib_kstat_KernelStat_statGetValue(
              JNIEnv  *env, 
              jobject obj, 
              jstring modName,
              jint instance,
              jstring instName,
              jstring propName
)
{
  const char    *mName;
  const char    *iName;
  const char    *pName;
  kstat_ctl_t   *kc;
  kstat_t       *ksp;
  kstat_named_t *kn;
  kstat_io_t    *ki;
  int           found;
  char          buf[BUFLEN];
  char          str[512];

  buf[0] = 0;

  /*
   * Process kstat open.
   */
  if ((kc = kstat_open()) == NULL) {
      TRACE("KSTAT(JNI)", LEVEL_ERROR, "kstat_open error : \n");
      return NULL;
  }
  
  /*
   * Convert JNI ==> C
   */
  if(modName == NULL)
      mName = NULL;
  else
      mName = (*env)->GetStringUTFChars(env, modName, 0);
  
  if(instName == NULL)
      iName = NULL;
  else
      iName = (*env)->GetStringUTFChars(env, instName, 0);
  
  if(propName == NULL)
      pName = NULL;
  else
      pName = (*env)->GetStringUTFChars(env, propName, 0);

  TRACE("KSTAT(JNI)", LEVEL_DEBUG, "statGetValue(%s,%d,%s,%s)\n",
             mName == NULL ? "NULL" : mName,
             instance,
             iName == NULL ? "NULL" : iName,
             pName == NULL ? "NULL" : pName);
  
  /*
   * Process the lookup
   */
  if ((ksp = kstat_lookup(kc,(char*)mName,instance,(char *)iName))==NULL) {
      TRACE("KSTAT(JNI)", LEVEL_ERROR, "kstat_lookup error : (%s,%d,%s) \n",
        (mName == NULL ? "NULL" : mName),
        instance,
        (iName == NULL ? "NULL" : iName));
    
    closeAndFreeModAndInstAndProp(env,
                  kc,
                  mName,
                  iName,
                  pName,
                  modName,
                  instName,
                  propName);

    return NULL;
  }  
  
  /*
   * Process the read.
   */
  if (kstat_read(kc, ksp, NULL) < 0) {
    TRACE("KSTAT(JNI)", LEVEL_ERROR, "kstat_read error : \n");
    closeAndFreeModAndInstAndProp(env,
                  kc,
                  mName,
                  iName,
                  pName,
                  modName,
                  instName,
                  propName);
    return NULL;
  }

  /*
   * Find out data type.
   */
  switch (ksp->ks_type) {
  case KSTAT_TYPE_RAW:
    TRACE("KSTAT(JNI)", LEVEL_DEBUG, "Data type KSTAT_TYPE_RAW: \n");
      sprintf(buf, "raw data not supported");
    break;
  case KSTAT_TYPE_NAMED:
      TRACE("KSTAT(JNI)", LEVEL_DEBUG, "Data type KSTAT_TYPE_NAMED: \n");
      /* Store the property */
      fillProperty(pName, kc, ksp, buf, &found);
      if (!found) {
      TRACE("KSTAT(JNI)", LEVEL_ERROR, 
        "Data type KSTAT_TYPE_NAMED: %s not found\n",
        pName);
      }
      closeAndFreeModAndInstAndProp(env,
                    kc,
                    mName,
                    iName,
                    pName,
                    modName,
                    instName,
                    propName);
      return (*env)->NewStringUTF(env, buf);
  case KSTAT_TYPE_INTR:
      TRACE("KSTAT(JNI)", LEVEL_DEBUG, 
        "Data type KSTAT_TYPE_INTR, not supported \n");
      sprintf(buf, "intr not supported");
      break;
  case KSTAT_TYPE_IO:
      TRACE("KSTAT(JNI)", LEVEL_DEBUG, "Data type KSTAT_TYPE_IO \n");
      ki = (kstat_io_t *)ksp->ks_data;
      
      if (strcmp(pName, "nread") == 0) {
      sprintf(buf, "%lld", ki->nread);
      break;
      }
      
      if (strcmp(pName, "nwritten") == 0) {
      sprintf(buf, "%lld", ki->nwritten);
      break;
      }
      
      if (strcmp(pName, "reads") == 0) {
      sprintf(buf, "%ld", ki->reads);
      break;
      }
      
      if (strcmp(pName, "writes") == 0) {
      sprintf(buf, "%ld", ki->writes);
      break;
      }
      
      if (strcmp(pName, "wtime") == 0) {
      sprintf(buf, "%lld", ki->wtime);
      break;
      }
      
      if (strcmp(pName, "wlentime") == 0) {
      sprintf(buf, "%lld", ki->wlentime);
      break;
      }
      
      if (strcmp(pName, "wlastupdate") == 0) {
      sprintf(buf, "%lld", ki->wlastupdate);
      break;
      }
      
      if (strcmp(pName, "rtime") == 0) {
      sprintf(buf, "%lld", ki->rtime);
      break;
      }
      
      if (strcmp(pName, "rlentime") == 0) {
      sprintf(buf, "%lld", ki->rlentime);
      break;
      }
      
      if (strcmp(pName, "rlastupdate") == 0) {
      sprintf(buf, "%lld", ki->rlastupdate);
      break;
      }
      
      if (strcmp(pName, "wcnt") == 0) {
      sprintf(buf, "%ld", ki->wcnt);
      break;
      }
      
      if (strcmp(pName, "rcnt") == 0) {
      sprintf(buf, "%ld", ki->rcnt);
      break;
      }
      
      closeAndFreeModAndInstAndProp(env,
                    kc,
                    mName,
                    iName,
                    pName,
                    modName,
                    instName,
                    propName);
      return NULL;
      break;
  case KSTAT_TYPE_TIMER:
    TRACE("KSTAT(JNI)", LEVEL_DEBUG, 
      "Data type KSTAT_TYPE_TIMER, not supported \n");
    sprintf(buf, "timer not supported");
    break;
  default:
    break;
  }

  /*
   * Close kstat.
   */
  closeAndFreeModAndInstAndProp(env,
                kc,
                mName,
                iName,
                pName,
                modName,
                instName,
                propName);
  /* 
   * Return a JNI string from the value (buf).
   */
  return ((*env)->NewStringUTF(env, buf));
}

/*
 * Class:     com_sun_jdmk_contrib_kstat_KernelStat
 * Method:    statProperties
 * Signature: (Ljava/lang/String;ILjava/lang/String;)Ljava/lang/String;
 *
 * Build a string of <properties name> of properties type for
 * this module and instance
 */
JNIEXPORT jstring JNICALL
Java_com_sun_jdmk_contrib_kstat_KernelStat_statProperties(JNIEnv  *env,
                              jobject obj,
                              jstring modName,
                              jint    instance,
                              jstring instName)
{
  jclass        cls = (*env)->GetObjectClass(env, obj);
  jfieldID      fid;
  jstring       jstr;
  jint          ji;
  const char    *mName;
  const char    *iName;
  int           ktype;
  char          buf[BUFLEN];
  kstat_ctl_t   *kc;
  kstat_t       *ksp;
  kstat_named_t *kn;
  int           i;
  int           len;
  char          *pos;

  mName = (*env)->GetStringUTFChars(env, modName, 0);
  iName = (*env)->GetStringUTFChars(env, instName, 0);

  if ((kc = kstat_open()) == NULL) {
    TRACE("KSTAT(JNI)", LEVEL_ERROR, "kstat_open error : \n");
    closeAndFreeModAndInst(env,
               kc,
               mName,
               iName,
               modName,
               instName);
    return (NULL);
  }
  
  if ((ksp=kstat_lookup(kc,(char*)mName,(int)instance,(char *)iName))==NULL) {
    TRACE("KSTAT(JNI)", LEVEL_ERROR, "kstat_lookup error : \n");
    closeAndFreeModAndInst(env,
               kc,
               mName,
               iName,
               modName,
               instName);
    return (NULL);
  }
  
  if (kstat_read(kc, ksp, NULL) < 0) {
    TRACE("KSTAT(JNI)", LEVEL_ERROR, "kstat_open read : \n");
    closeAndFreeModAndInst(env,
               kc,
               mName,
               iName,
               modName,
               instName);
    return (NULL);
  }

  buf[0] = 0;
  len    = 0;

  switch (ksp->ks_type) {
  case KSTAT_TYPE_RAW:
    sprintf(buf, "rawData:RawData");
    break;
  case KSTAT_TYPE_NAMED:
    kn = (kstat_named_t *)ksp->ks_data;
    for (i = 0; i < ksp->ks_ndata; i++) {
      char knn[KSTAT_STRLEN+1];
      knn[0]='\0';
      strncpy(knn,kn->name,KSTAT_STRLEN);
      knn[KSTAT_STRLEN]='\0';

      if (buf[0] != 0) {
        strcat(buf, " ");
        len = strlen(buf);
      }
      if (len > (BUFLEN - (strlen(knn) + 17))) {
    TRACE("KSTAT(JNI)", LEVEL_ERROR, "Buffer not big enough. \n");
    break;
      }
      pos = &buf[len];
      switch (kn->data_type) {
      case KSTAT_DATA_CHAR:
        sprintf(pos, "%s:DataChar", knn);
        break;
      case KSTAT_DATA_INT32:
        sprintf(pos, "%s:Integer", knn);
        break;
      case KSTAT_DATA_UINT32:
        sprintf(pos, "%s:UnsignedInteger", knn);
        break;
      case KSTAT_DATA_INT64:
        sprintf(pos, "%s:Long", knn);
        break;
      case KSTAT_DATA_UINT64:
        sprintf(pos, "%s:UnsignedLong", knn);
        break;
      default:
        sprintf(pos, "%s:Unknown", knn);
        break;
      }
      kn++;
    }
    break;
  case KSTAT_TYPE_INTR:
    sprintf(buf, "intrData:IntrData");
    break;
  case KSTAT_TYPE_IO:
    sprintf(buf, 
        "nread:UnsignedLong "
        "nwritten:UnsignedLong "
        "reads:UnsignedInteger "
        "writes:UnsignedInteger "
        "wtime:UnsignedLong "
        "wlentime:UnsignedLong "
        "wlastupdate:UnsignedLong "
        "rtime:UnsignedLong "
        "rlentime:UnsignedLong "
        "rlastupdate:UnsignedLong "
        "wcnt:UnsignedInteger "
        "rcnt:UnsignedInteger");
    break;
  case KSTAT_TYPE_TIMER:
    sprintf(buf, "timerData:TimerData");
    break;
  default:
    break;
  }

  closeAndFreeModAndInst(env,
             kc,
             mName,
             iName,
             modName,
             instName);

  return ((*env)->NewStringUTF(env, buf));
}

/*
 * Class:     com_sun_jdmk_contrib_kstat_KernelStat
 * Method:    statType
 * Signature: (Ljava/lang/String;ILjava/lang/String;)I;
 */
JNIEXPORT jint JNICALL
Java_com_sun_jdmk_contrib_kstat_KernelStat_statType(JNIEnv  *env,
                            jobject obj,
                            jstring modName,
                            jint    instance,
                            jstring instName)
{
  jclass      cls = (*env)->GetObjectClass(env, obj);
  jfieldID    fid;
  jstring     jstr;
  const char  *mName;
  const char  *iName;
  int         type;
  kstat_ctl_t *kc;
  kstat_t     *ksp;

  mName = (*env)->GetStringUTFChars(env, modName, 0);
  iName = (*env)->GetStringUTFChars(env, instName, 0);
  TRACE("KSTAT(JNI)", LEVEL_DEBUG,
    "Looking for type of module-name [%s] instance [%d] instance-name [%s]\n", 
    mName, iName);

  if ((kc = kstat_open()) == NULL) {
    TRACE("KSTAT(JNI)", LEVEL_ERROR, "kstat_open error : \n");
    type = -1;
    closeAndFreeModAndInst(env,
               kc,
               mName,
               iName,
               modName,
               instName);
    return (type);
  }

  if ((ksp = kstat_lookup(kc, (char*)mName, instance, 
              (char *)iName)) == NULL) {
    TRACE("KSTAT(JNI)", LEVEL_ERROR, "kstat_lookup error : \n");
    type = -1;
    closeAndFreeModAndInst(env,
               kc,
               mName,
               iName,
               modName,
               instName);
    return (type);
  }
  
  type = ksp->ks_type;
  closeAndFreeModAndInst(env,
             kc,
             mName,
             iName,
             modName,
             instName);
  
  return (type);
}
